Package: fauxpas
Title: HTTP Error Helpers
Description: HTTP error helpers. Methods included for general purpose HTTP
    error handling, as well as individual methods for every HTTP status
    code, both via status code numbers as well as their descriptive names.
    Supports ability to adjust behavior to stop, message or warning.
    Includes ability to use custom whisker template to have any configuration
    of status code, short description, and verbose message. Currently
    supports integration with 'crul', 'curl', and 'httr'.
Version: 0.6.0
Authors@R: person("Scott", "Chamberlain", role = c("aut", "cre"),
    email = "myrmecocystus@gmail.com",
    comment = c(ORCID = "0000-0003-1444-9135"))
License: MIT + file LICENSE
URL: https://sckott.github.io/fauxpas/,
        https://github.com/sckott/fauxpas
BugReports: https://github.com/sckott/fauxpas/issues
VignetteBuilder: knitr
Encoding: UTF-8
Language: en-US
Imports: R6 (>= 2.1.2), httpcode (>= 0.3.0), whisker
Suggests: crul (>= 0.5.0), curl (>= 2.2), httr (>= 1.2.0), testthat,
        knitr, rmarkdown
RoxygenNote: 7.3.3
X-schema.org-applicationCategory: Web
X-schema.org-keywords: http, https, API, web-services, curl, errors,
        error
NeedsCompilation: no
Packaged: 2026-02-19 23:57:43 UTC; sckott
Author: Scott Chamberlain [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-1444-9135>)
Maintainer: Scott Chamberlain <myrmecocystus@gmail.com>
Repository: CRAN
Date/Publication: 2026-02-20 06:40:10 UTC
Built: R 4.6.0; ; 2026-04-25 17:36:22 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(http,HttpResponse)
S3method(http,default)
S3method(http,list)
S3method(http,response)
S3method(http100,HttpResponse)
S3method(http100,default)
S3method(http100,list)
S3method(http100,response)
S3method(http101,HttpResponse)
S3method(http101,default)
S3method(http101,list)
S3method(http101,response)
S3method(http102,HttpResponse)
S3method(http102,default)
S3method(http102,list)
S3method(http102,response)
S3method(http200,HttpResponse)
S3method(http200,default)
S3method(http200,list)
S3method(http200,response)
S3method(http201,HttpResponse)
S3method(http201,default)
S3method(http201,list)
S3method(http201,response)
S3method(http202,HttpResponse)
S3method(http202,default)
S3method(http202,list)
S3method(http202,response)
S3method(http203,HttpResponse)
S3method(http203,default)
S3method(http203,list)
S3method(http203,response)
S3method(http204,HttpResponse)
S3method(http204,default)
S3method(http204,list)
S3method(http204,response)
S3method(http205,HttpResponse)
S3method(http205,default)
S3method(http205,list)
S3method(http205,response)
S3method(http206,HttpResponse)
S3method(http206,default)
S3method(http206,list)
S3method(http206,response)
S3method(http207,HttpResponse)
S3method(http207,default)
S3method(http207,list)
S3method(http207,response)
S3method(http208,HttpResponse)
S3method(http208,default)
S3method(http208,list)
S3method(http208,response)
S3method(http226,HttpResponse)
S3method(http226,default)
S3method(http226,list)
S3method(http226,response)
S3method(http300,HttpResponse)
S3method(http300,default)
S3method(http300,list)
S3method(http300,response)
S3method(http301,HttpResponse)
S3method(http301,default)
S3method(http301,list)
S3method(http301,response)
S3method(http302,HttpResponse)
S3method(http302,default)
S3method(http302,list)
S3method(http302,response)
S3method(http303,HttpResponse)
S3method(http303,default)
S3method(http303,list)
S3method(http303,response)
S3method(http304,HttpResponse)
S3method(http304,default)
S3method(http304,list)
S3method(http304,response)
S3method(http305,HttpResponse)
S3method(http305,default)
S3method(http305,list)
S3method(http305,response)
S3method(http306,HttpResponse)
S3method(http306,default)
S3method(http306,list)
S3method(http306,response)
S3method(http307,HttpResponse)
S3method(http307,default)
S3method(http307,list)
S3method(http307,response)
S3method(http308,HttpResponse)
S3method(http308,default)
S3method(http308,list)
S3method(http308,response)
S3method(http400,HttpResponse)
S3method(http400,default)
S3method(http400,list)
S3method(http400,response)
S3method(http401,HttpResponse)
S3method(http401,default)
S3method(http401,list)
S3method(http401,response)
S3method(http402,HttpResponse)
S3method(http402,default)
S3method(http402,list)
S3method(http402,response)
S3method(http403,HttpResponse)
S3method(http403,default)
S3method(http403,list)
S3method(http403,response)
S3method(http404,HttpResponse)
S3method(http404,default)
S3method(http404,list)
S3method(http404,response)
S3method(http405,HttpResponse)
S3method(http405,default)
S3method(http405,list)
S3method(http405,response)
S3method(http406,HttpResponse)
S3method(http406,default)
S3method(http406,list)
S3method(http406,response)
S3method(http407,HttpResponse)
S3method(http407,default)
S3method(http407,list)
S3method(http407,response)
S3method(http408,HttpResponse)
S3method(http408,default)
S3method(http408,list)
S3method(http408,response)
S3method(http409,HttpResponse)
S3method(http409,default)
S3method(http409,list)
S3method(http409,response)
S3method(http410,HttpResponse)
S3method(http410,default)
S3method(http410,list)
S3method(http410,response)
S3method(http411,HttpResponse)
S3method(http411,default)
S3method(http411,list)
S3method(http411,response)
S3method(http412,HttpResponse)
S3method(http412,default)
S3method(http412,list)
S3method(http412,response)
S3method(http413,HttpResponse)
S3method(http413,default)
S3method(http413,list)
S3method(http413,response)
S3method(http414,HttpResponse)
S3method(http414,default)
S3method(http414,list)
S3method(http414,response)
S3method(http415,HttpResponse)
S3method(http415,default)
S3method(http415,list)
S3method(http415,response)
S3method(http416,HttpResponse)
S3method(http416,default)
S3method(http416,list)
S3method(http416,response)
S3method(http417,HttpResponse)
S3method(http417,default)
S3method(http417,list)
S3method(http417,response)
S3method(http418,HttpResponse)
S3method(http418,default)
S3method(http418,list)
S3method(http418,response)
S3method(http419,HttpResponse)
S3method(http419,default)
S3method(http419,list)
S3method(http419,response)
S3method(http420,HttpResponse)
S3method(http420,default)
S3method(http420,list)
S3method(http420,response)
S3method(http421,HttpResponse)
S3method(http421,default)
S3method(http421,list)
S3method(http421,response)
S3method(http422,HttpResponse)
S3method(http422,default)
S3method(http422,list)
S3method(http422,response)
S3method(http423,HttpResponse)
S3method(http423,default)
S3method(http423,list)
S3method(http423,response)
S3method(http424,HttpResponse)
S3method(http424,default)
S3method(http424,list)
S3method(http424,response)
S3method(http425,HttpResponse)
S3method(http425,default)
S3method(http425,list)
S3method(http425,response)
S3method(http426,HttpResponse)
S3method(http426,default)
S3method(http426,list)
S3method(http426,response)
S3method(http428,HttpResponse)
S3method(http428,default)
S3method(http428,list)
S3method(http428,response)
S3method(http429,HttpResponse)
S3method(http429,default)
S3method(http429,list)
S3method(http429,response)
S3method(http431,HttpResponse)
S3method(http431,default)
S3method(http431,list)
S3method(http431,response)
S3method(http440,HttpResponse)
S3method(http440,default)
S3method(http440,list)
S3method(http440,response)
S3method(http444,HttpResponse)
S3method(http444,default)
S3method(http444,list)
S3method(http444,response)
S3method(http449,HttpResponse)
S3method(http449,default)
S3method(http449,list)
S3method(http449,response)
S3method(http450,HttpResponse)
S3method(http450,default)
S3method(http450,list)
S3method(http450,response)
S3method(http451,HttpResponse)
S3method(http451,default)
S3method(http451,list)
S3method(http451,response)
S3method(http494,HttpResponse)
S3method(http494,default)
S3method(http494,list)
S3method(http494,response)
S3method(http495,HttpResponse)
S3method(http495,default)
S3method(http495,list)
S3method(http495,response)
S3method(http496,HttpResponse)
S3method(http496,default)
S3method(http496,list)
S3method(http496,response)
S3method(http497,HttpResponse)
S3method(http497,default)
S3method(http497,list)
S3method(http497,response)
S3method(http498,HttpResponse)
S3method(http498,default)
S3method(http498,list)
S3method(http498,response)
S3method(http499,HttpResponse)
S3method(http499,default)
S3method(http499,list)
S3method(http499,response)
S3method(http500,HttpResponse)
S3method(http500,default)
S3method(http500,list)
S3method(http500,response)
S3method(http501,HttpResponse)
S3method(http501,default)
S3method(http501,list)
S3method(http501,response)
S3method(http502,HttpResponse)
S3method(http502,default)
S3method(http502,list)
S3method(http502,response)
S3method(http503,HttpResponse)
S3method(http503,default)
S3method(http503,list)
S3method(http503,response)
S3method(http504,HttpResponse)
S3method(http504,default)
S3method(http504,list)
S3method(http504,response)
S3method(http505,HttpResponse)
S3method(http505,default)
S3method(http505,list)
S3method(http505,response)
S3method(http506,HttpResponse)
S3method(http506,default)
S3method(http506,list)
S3method(http506,response)
S3method(http507,HttpResponse)
S3method(http507,default)
S3method(http507,list)
S3method(http507,response)
S3method(http508,HttpResponse)
S3method(http508,default)
S3method(http508,list)
S3method(http508,response)
S3method(http509,HttpResponse)
S3method(http509,default)
S3method(http509,list)
S3method(http509,response)
S3method(http510,HttpResponse)
S3method(http510,default)
S3method(http510,list)
S3method(http510,response)
S3method(http511,HttpResponse)
S3method(http511,default)
S3method(http511,list)
S3method(http511,response)
S3method(http520,HttpResponse)
S3method(http520,default)
S3method(http520,list)
S3method(http520,response)
S3method(http521,HttpResponse)
S3method(http521,default)
S3method(http521,list)
S3method(http521,response)
S3method(http522,HttpResponse)
S3method(http522,default)
S3method(http522,list)
S3method(http522,response)
S3method(http523,HttpResponse)
S3method(http523,default)
S3method(http523,list)
S3method(http523,response)
S3method(http524,HttpResponse)
S3method(http524,default)
S3method(http524,list)
S3method(http524,response)
S3method(http525,HttpResponse)
S3method(http525,default)
S3method(http525,list)
S3method(http525,response)
S3method(http526,HttpResponse)
S3method(http526,default)
S3method(http526,list)
S3method(http526,response)
S3method(http527,HttpResponse)
S3method(http527,default)
S3method(http527,list)
S3method(http527,response)
S3method(http598,HttpResponse)
S3method(http598,default)
S3method(http598,list)
S3method(http598,response)
S3method(http599,HttpResponse)
S3method(http599,default)
S3method(http599,list)
S3method(http599,response)
export(Error)
export(HTTPATimeoutOccurred)
export(HTTPAccepted)
export(HTTPAlreadyReported)
export(HTTPAuthenticationTimeout)
export(HTTPBadGateway)
export(HTTPBadRequest)
export(HTTPBandwidthLimitExceeded)
export(HTTPBlockedByWindowsParentalControls)
export(HTTPCertError)
export(HTTPClientClosedRequest)
export(HTTPConflict)
export(HTTPConnectionTimedOut)
export(HTTPContinue)
export(HTTPCreated)
export(HTTPExpectationFailed)
export(HTTPFailedDependency)
export(HTTPForbidden)
export(HTTPFound)
export(HTTPGatewayTimeout)
export(HTTPGone)
export(HTTPHTTPToHTTPS)
export(HTTPHTTPVersionNotSupported)
export(HTTPImUsed)
export(HTTPInsufficientStorage)
export(HTTPInternalServerError)
export(HTTPInvalidSSLCertificate)
export(HTTPLengthRequired)
export(HTTPLocked)
export(HTTPLoginTimeout)
export(HTTPLoopDetected)
export(HTTPMethodFailure)
export(HTTPMethodNotAllowed)
export(HTTPMisdirectedRequest)
export(HTTPMovedPermanently)
export(HTTPMultiStatus)
export(HTTPMultipleChoices)
export(HTTPNetworkAuthenticationRequired)
export(HTTPNetworkConnectTimeoutError)
export(HTTPNetworkReadTimeoutError)
export(HTTPNoCert)
export(HTTPNoContent)
export(HTTPNoResponse)
export(HTTPNonAuthoritativeInformation)
export(HTTPNotAcceptable)
export(HTTPNotExtended)
export(HTTPNotFound)
export(HTTPNotImplemented)
export(HTTPNotModified)
export(HTTPOK)
export(HTTPOriginIsUnreachable)
export(HTTPPartialContent)
export(HTTPPaymentRequired)
export(HTTPPermanentRedirect)
export(HTTPPreconditionFailed)
export(HTTPPreconditionRequired)
export(HTTPProcessing)
export(HTTPProxyAuthenticationRequired)
export(HTTPRailgunError)
export(HTTPRequestEntityTooLarge)
export(HTTPRequestHeaderFieldsTooLarge)
export(HTTPRequestHeaderTooLarge)
export(HTTPRequestRangeNotSatisfiable)
export(HTTPRequestTimeout)
export(HTTPRequestURITooLong)
export(HTTPResetContent)
export(HTTPRetryWith)
export(HTTPSSLHandshakeFailed)
export(HTTPSeeOther)
export(HTTPServiceUnavailable)
export(HTTPSwitchProtocol)
export(HTTPSwitchProxy)
export(HTTPTeaPot)
export(HTTPTemporaryRedirect)
export(HTTPTokenExpiredInvalid)
export(HTTPTooManyRequests)
export(HTTPUnauthorized)
export(HTTPUnavailableForLegalReasons)
export(HTTPUnorderedCollection)
export(HTTPUnprocessableEntity)
export(HTTPUnsupportedMediaType)
export(HTTPUpgradeRequired)
export(HTTPUseProxy)
export(HTTPVariantAlsoNegotiates)
export(HTTPWebServerIsDown)
export(HTTPWebServerReturnedUnknownError)
export(find_error_class)
export(http)
export(http100)
export(http101)
export(http102)
export(http200)
export(http201)
export(http202)
export(http203)
export(http204)
export(http205)
export(http206)
export(http207)
export(http208)
export(http226)
export(http300)
export(http301)
export(http302)
export(http303)
export(http304)
export(http305)
export(http306)
export(http307)
export(http308)
export(http400)
export(http401)
export(http402)
export(http403)
export(http404)
export(http405)
export(http406)
export(http407)
export(http408)
export(http409)
export(http410)
export(http411)
export(http412)
export(http413)
export(http414)
export(http415)
export(http416)
export(http417)
export(http418)
export(http419)
export(http420)
export(http421)
export(http422)
export(http423)
export(http424)
export(http425)
export(http426)
export(http428)
export(http429)
export(http431)
export(http440)
export(http444)
export(http449)
export(http450)
export(http451)
export(http494)
export(http495)
export(http496)
export(http497)
export(http498)
export(http499)
export(http500)
export(http501)
export(http502)
export(http503)
export(http504)
export(http505)
export(http506)
export(http507)
export(http508)
export(http509)
export(http510)
export(http511)
export(http520)
export(http521)
export(http522)
export(http523)
export(http524)
export(http525)
export(http526)
export(http527)
export(http598)
export(http599)
importFrom(R6,R6Class)
importFrom(httpcode,http_code)
importFrom(whisker,whisker.render)

fauxpas
=======



[![R-check](https://github.com/sckott/fauxpas/actions/workflows/R-check.yml/badge.svg?branch=main)](https://github.com/sckott/fauxpas/actions/workflows/R-check.yml)
[![cran checks](https://badges.cranchecks.info/worst/fauxpas.svg)](https://github.com/sckott/fauxpas)
[![cran version](http://www.r-pkg.org/badges/version/fauxpas)](https://cran.r-project.org/package=fauxpas)
[![rstudio mirror downloads](https://cranlogs.r-pkg.org/badges/fauxpas)](https://github.com/r-hub/cranlogs.app)

`fauxpas` does http errors

* HTTP error classes more in line with Ruby/Python/Etc.
* An error class for each HTTP status in case a user wants to
be specific to an HTTP status code, and general purpose handlers
for any error
* Work with any of the major R http clients: `crul`, `curl`, `httr`, (maybe
`RCurl` later)
* Provide flexiblity for what to do on an HTTP error, including
custom functions and message templates

Info Links:

* HTTP on wikipedia: <https://en.wikipedia.org/wiki/Hypertext_Transfer_Protocol>
* HTTP2 on wikipedia: <https://en.wikipedia.org/wiki/HTTP/2>
* HTTP/1.1: <https://www.w3.org/Protocols/rfc2616/rfc2616-sec10.html>
* HTTP/2: <https://datatracker.ietf.org/doc/html/rfc7540>

## Install

CRAN version


``` r
# install.packages("pak")
pak::pak("fauxpas")
```

Dev version


``` r
pak::pak("sckott/fauxpas")
```


``` r
library("fauxpas")
```

## use with crul


``` r
library("crul")
cli <- HttpClient$new("https://httpbin.org/status/414")
res <- cli$get()
http(res)
#> Error:
#> ! Request-URI Too Long (HTTP 414)
http414(res)
#> Error:
#> ! Request-URI Too Long (HTTP 414)
```


``` r
x <- HTTPRequestURITooLong$new()
x$do_verbose(res)
#> Error:
#> ! Request-URI Too Long (HTTP 414).
#>  - The server is refusing to service the request because the Request-URI is longer than the server is willing to interpret. This rare condition is only likely to occur when a client has improperly converted a POST request to a GET request with long query information, when the client has descended into a URI 'black hole' of redirection (e.g., a redirected URI prefix that points to a suffix of itself), or when the server is under attack by a client attempting to exploit security holes present in some servers using fixed-length buffers for reading or manipulating the Request-URI.
```

## use with curl


``` r
library("curl")
h <- curl::new_handle()
curl::handle_setopt(h)
resp <- curl::curl_fetch_memory("https://httpbin.org/status/404", h)
http(resp)
#> Error:
#> ! Not Found (HTTP 404)
http404(resp)
#> Error:
#> ! Not Found (HTTP 404)
```


``` r
x <- HTTPNotFound$new()
x$do_verbose(resp)
#> Error:
#> ! Not Found (HTTP 404).
#>  - The server has not found anything matching the Request-URI. No indication is given of whether the condition is temporary or permanent. The 410 (Gone) status code SHOULD be used if the server knows, through some internally configurable mechanism, that an old resource is permanently unavailable and has no forwarding address. This status code is commonly used when the server does not wish to reveal exactly why the request has been refused, or when no other response is applicable.
```

## use with httr


``` r
library("httr")
res <- GET("https://httpbin.org/status/405")
http405(res)
#> Error:
#> ! Method Not Allowed (HTTP 405)
```


``` r
x <- HTTPMethodNotAllowed$new()
x$do_verbose(res)
#> Error:
#> ! Method Not Allowed (HTTP 405).
#>  - The method specified in the Request-Line is not allowed for the resource identified by the Request-URI. The response MUST include an Allow header containing a list of valid methods for the requested resource.
```

## Meta

* Please [report any issues or bugs](https://github.com/sckott/fauxpas/issues)
* License: MIT
* Get citation information for `fauxpas` in R doing `citation(package = 'fauxpas')`
* Please note that the fauxpas project is released with a [Contributor Code of Conduct](https://contributor-covenant.org/version/2/1/CODE_OF_CONDUCT.html). By contributing to this project, you agree to abide by its terms.

## ----echo=FALSE---------------------------------------------------------------
knitr::opts_chunk$set(
  comment = "#>",
  collapse = TRUE,
  warning = FALSE,
  message = FALSE
)

## ----eval=FALSE---------------------------------------------------------------
# install.packages("fauxpas")

## ----eval=FALSE---------------------------------------------------------------
# # install.packages("remotes")
# remotes::install_github("sckott/fauxpas")

## -----------------------------------------------------------------------------
library("fauxpas")

## -----------------------------------------------------------------------------
(x <- find_error_class(418))

## -----------------------------------------------------------------------------
x$new()

## ----eval=FALSE---------------------------------------------------------------
# find_error_class(999)
# #> Error in find_error_class(999) : no method found for 999

## ----eval=FALSE---------------------------------------------------------------
# library("crul")
# cli <- HttpClient$new("https://httpbin.org/status/414")
# res <- cli$get()
# http(res)
# #> Error: Request-URI Too Long (HTTP 414).
# http414(res)
# #> Error: Request-URI Too Long (HTTP 414).

## ----eval=FALSE---------------------------------------------------------------
# x <- HTTPRequestURITooLong$new()
# x$do_verbose(res)
# #> Error: Request-URI Too Long (HTTP 414).
# #> - The server is refusing to service the request because the Request-URI is
# #>    longer than the server is willing to interpret. This rare condition is only likely
# #>    to occur when a client has improperly converted a POST request to a GET request
# #>    with long query information, when the client has descended into a URI black hole
# #>    of redirection (e.g., a redirected URI prefix that points to a suffix of itself),
# #>    or when the server is under attack by a client attempting to exploit security
# #>    holes present in some servers using fixed-length buffers for reading or
# #>    manipulating the Request-URI.

## ----eval=FALSE---------------------------------------------------------------
# library("curl")
# h <- curl::new_handle()
# curl::handle_setopt(h)
# resp <- curl::curl_fetch_memory("https://httpbin.org/status/404", h)
# http(resp)
# #> Error: Not Found (HTTP 404).
# http404(resp)
# #> Error: Not Found (HTTP 404).

## ----eval=FALSE---------------------------------------------------------------
# x <- HTTPNotFound$new()
# x$do_verbose(resp)
# #> Error:  Not Found (HTTP 404).
# #>  - The server has not found anything matching the Request-URI. No indication is
# #> given of whether the condition is temporary or permanent. The 410 (Gone) status
# #> code SHOULD be used if the server knows, through some internally configurable
# #> mechanism, that an old resource is permanently unavailable and has no forwarding
# #> address. #> This status code is commonly used when the server does not wish to
# #> reveal exactly why the request has been refused, or when no other response is
# #> applicable.

## ----eval=FALSE---------------------------------------------------------------
# library("httr")
# res <- GET("https://httpbin.org/status/405")
# http405(res)
# #> Error: Method Not Allowed (HTTP 405).

## ----eval=FALSE---------------------------------------------------------------
# x <- HTTPMethodNotAllowed$new()
# x$do_verbose(res)
# #> Error: Method Not Allowed (HTTP 405).
# #>  - The method specified in the Request-Line is not allowed for the resource
# #> identified by the Request-URI. The response MUST include an Allow header
# #> containing a list of valid methods for the requested resource.

# generated from extra/generate.R

http100_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPContinue$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http100 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http100")
}
#' @export
http100.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http100' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http100.response <- http100_
#' @export
http100.HttpResponse <- http100_
#' @export
http100.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http100_(response, behavior, message_template)
}

http101_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPSwitchProtocol$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http101 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http101")
}
#' @export
http101.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http101' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http101.response <- http101_
#' @export
http101.HttpResponse <- http101_
#' @export
http101.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http101_(response, behavior, message_template)
}

http102_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPProcessing$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http102 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http102")
}
#' @export
http102.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http102' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http102.response <- http102_
#' @export
http102.HttpResponse <- http102_
#' @export
http102.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http102_(response, behavior, message_template)
}

http200_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPOK$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http200 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http200")
}
#' @export
http200.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http200' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http200.response <- http200_
#' @export
http200.HttpResponse <- http200_
#' @export
http200.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http200_(response, behavior, message_template)
}

http201_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPCreated$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http201 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http201")
}
#' @export
http201.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http201' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http201.response <- http201_
#' @export
http201.HttpResponse <- http201_
#' @export
http201.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http201_(response, behavior, message_template)
}

http202_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPAccepted$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http202 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http202")
}
#' @export
http202.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http202' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http202.response <- http202_
#' @export
http202.HttpResponse <- http202_
#' @export
http202.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http202_(response, behavior, message_template)
}

http203_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPNonAuthoritativeInformation$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http203 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http203")
}
#' @export
http203.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http203' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http203.response <- http203_
#' @export
http203.HttpResponse <- http203_
#' @export
http203.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http203_(response, behavior, message_template)
}

http204_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPNoContent$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http204 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http204")
}
#' @export
http204.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http204' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http204.response <- http204_
#' @export
http204.HttpResponse <- http204_
#' @export
http204.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http204_(response, behavior, message_template)
}

http205_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPResetContent$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http205 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http205")
}
#' @export
http205.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http205' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http205.response <- http205_
#' @export
http205.HttpResponse <- http205_
#' @export
http205.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http205_(response, behavior, message_template)
}

http206_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPPartialContent$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http206 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http206")
}
#' @export
http206.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http206' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http206.response <- http206_
#' @export
http206.HttpResponse <- http206_
#' @export
http206.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http206_(response, behavior, message_template)
}

http207_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPMultiStatus$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http207 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http207")
}
#' @export
http207.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http207' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http207.response <- http207_
#' @export
http207.HttpResponse <- http207_
#' @export
http207.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http207_(response, behavior, message_template)
}

http208_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPAlreadyReported$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http208 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http208")
}
#' @export
http208.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http208' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http208.response <- http208_
#' @export
http208.HttpResponse <- http208_
#' @export
http208.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http208_(response, behavior, message_template)
}

http226_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPImUsed$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http226 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http226")
}
#' @export
http226.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http226' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http226.response <- http226_
#' @export
http226.HttpResponse <- http226_
#' @export
http226.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http226_(response, behavior, message_template)
}

http300_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPMultipleChoices$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http300 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http300")
}
#' @export
http300.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http300' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http300.response <- http300_
#' @export
http300.HttpResponse <- http300_
#' @export
http300.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http300_(response, behavior, message_template)
}

http301_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPMovedPermanently$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http301 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http301")
}
#' @export
http301.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http301' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http301.response <- http301_
#' @export
http301.HttpResponse <- http301_
#' @export
http301.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http301_(response, behavior, message_template)
}

http302_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPFound$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http302 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http302")
}
#' @export
http302.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http302' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http302.response <- http302_
#' @export
http302.HttpResponse <- http302_
#' @export
http302.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http302_(response, behavior, message_template)
}

http303_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPSeeOther$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http303 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http303")
}
#' @export
http303.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http303' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http303.response <- http303_
#' @export
http303.HttpResponse <- http303_
#' @export
http303.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http303_(response, behavior, message_template)
}

http304_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPNotModified$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http304 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http304")
}
#' @export
http304.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http304' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http304.response <- http304_
#' @export
http304.HttpResponse <- http304_
#' @export
http304.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http304_(response, behavior, message_template)
}

http305_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPUseProxy$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http305 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http305")
}
#' @export
http305.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http305' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http305.response <- http305_
#' @export
http305.HttpResponse <- http305_
#' @export
http305.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http305_(response, behavior, message_template)
}

http306_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPSwitchProxy$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http306 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http306")
}
#' @export
http306.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http306' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http306.response <- http306_
#' @export
http306.HttpResponse <- http306_
#' @export
http306.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http306_(response, behavior, message_template)
}

http307_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPTemporaryRedirect$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http307 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http307")
}
#' @export
http307.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http307' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http307.response <- http307_
#' @export
http307.HttpResponse <- http307_
#' @export
http307.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http307_(response, behavior, message_template)
}

http308_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPPermanentRedirect$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http308 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http308")
}
#' @export
http308.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http308' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http308.response <- http308_
#' @export
http308.HttpResponse <- http308_
#' @export
http308.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http308_(response, behavior, message_template)
}

http400_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPBadRequest$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http400 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http400")
}
#' @export
http400.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http400' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http400.response <- http400_
#' @export
http400.HttpResponse <- http400_
#' @export
http400.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http400_(response, behavior, message_template)
}

http401_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPUnauthorized$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http401 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http401")
}
#' @export
http401.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http401' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http401.response <- http401_
#' @export
http401.HttpResponse <- http401_
#' @export
http401.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http401_(response, behavior, message_template)
}

http402_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPPaymentRequired$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http402 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http402")
}
#' @export
http402.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http402' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http402.response <- http402_
#' @export
http402.HttpResponse <- http402_
#' @export
http402.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http402_(response, behavior, message_template)
}

http403_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPForbidden$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http403 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http403")
}
#' @export
http403.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http403' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http403.response <- http403_
#' @export
http403.HttpResponse <- http403_
#' @export
http403.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http403_(response, behavior, message_template)
}

http404_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPNotFound$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http404 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http404")
}
#' @export
http404.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http404' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http404.response <- http404_
#' @export
http404.HttpResponse <- http404_
#' @export
http404.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http404_(response, behavior, message_template)
}

http405_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPMethodNotAllowed$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http405 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http405")
}
#' @export
http405.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http405' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http405.response <- http405_
#' @export
http405.HttpResponse <- http405_
#' @export
http405.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http405_(response, behavior, message_template)
}

http406_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPNotAcceptable$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http406 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http406")
}
#' @export
http406.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http406' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http406.response <- http406_
#' @export
http406.HttpResponse <- http406_
#' @export
http406.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http406_(response, behavior, message_template)
}

http407_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPProxyAuthenticationRequired$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http407 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http407")
}
#' @export
http407.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http407' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http407.response <- http407_
#' @export
http407.HttpResponse <- http407_
#' @export
http407.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http407_(response, behavior, message_template)
}

http408_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPRequestTimeout$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http408 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http408")
}
#' @export
http408.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http408' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http408.response <- http408_
#' @export
http408.HttpResponse <- http408_
#' @export
http408.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http408_(response, behavior, message_template)
}

http409_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPConflict$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http409 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http409")
}
#' @export
http409.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http409' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http409.response <- http409_
#' @export
http409.HttpResponse <- http409_
#' @export
http409.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http409_(response, behavior, message_template)
}

http410_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPGone$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http410 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http410")
}
#' @export
http410.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http410' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http410.response <- http410_
#' @export
http410.HttpResponse <- http410_
#' @export
http410.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http410_(response, behavior, message_template)
}

http411_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPLengthRequired$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http411 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http411")
}
#' @export
http411.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http411' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http411.response <- http411_
#' @export
http411.HttpResponse <- http411_
#' @export
http411.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http411_(response, behavior, message_template)
}

http412_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPPreconditionFailed$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http412 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http412")
}
#' @export
http412.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http412' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http412.response <- http412_
#' @export
http412.HttpResponse <- http412_
#' @export
http412.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http412_(response, behavior, message_template)
}

http413_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPRequestEntityTooLarge$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http413 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http413")
}
#' @export
http413.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http413' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http413.response <- http413_
#' @export
http413.HttpResponse <- http413_
#' @export
http413.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http413_(response, behavior, message_template)
}

http414_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPRequestURITooLong$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http414 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http414")
}
#' @export
http414.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http414' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http414.response <- http414_
#' @export
http414.HttpResponse <- http414_
#' @export
http414.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http414_(response, behavior, message_template)
}

http415_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPUnsupportedMediaType$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http415 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http415")
}
#' @export
http415.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http415' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http415.response <- http415_
#' @export
http415.HttpResponse <- http415_
#' @export
http415.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http415_(response, behavior, message_template)
}

http416_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPRequestRangeNotSatisfiable$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http416 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http416")
}
#' @export
http416.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http416' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http416.response <- http416_
#' @export
http416.HttpResponse <- http416_
#' @export
http416.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http416_(response, behavior, message_template)
}

http417_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPExpectationFailed$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http417 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http417")
}
#' @export
http417.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http417' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http417.response <- http417_
#' @export
http417.HttpResponse <- http417_
#' @export
http417.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http417_(response, behavior, message_template)
}

http418_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPTeaPot$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http418 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http418")
}
#' @export
http418.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http418' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http418.response <- http418_
#' @export
http418.HttpResponse <- http418_
#' @export
http418.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http418_(response, behavior, message_template)
}

http419_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPAuthenticationTimeout$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http419 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http419")
}
#' @export
http419.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http419' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http419.response <- http419_
#' @export
http419.HttpResponse <- http419_
#' @export
http419.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http419_(response, behavior, message_template)
}

http420_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPMethodFailure$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http420 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http420")
}
#' @export
http420.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http420' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http420.response <- http420_
#' @export
http420.HttpResponse <- http420_
#' @export
http420.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http420_(response, behavior, message_template)
}

http422_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPUnprocessableEntity$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http422 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http422")
}
#' @export
http422.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http422' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http422.response <- http422_
#' @export
http422.HttpResponse <- http422_
#' @export
http422.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http422_(response, behavior, message_template)
}

http423_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPLocked$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http423 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http423")
}
#' @export
http423.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http423' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http423.response <- http423_
#' @export
http423.HttpResponse <- http423_
#' @export
http423.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http423_(response, behavior, message_template)
}

http424_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPFailedDependency$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http424 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http424")
}
#' @export
http424.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http424' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http424.response <- http424_
#' @export
http424.HttpResponse <- http424_
#' @export
http424.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http424_(response, behavior, message_template)
}

http425_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPUnorderedCollection$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http425 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http425")
}
#' @export
http425.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http425' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http425.response <- http425_
#' @export
http425.HttpResponse <- http425_
#' @export
http425.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http425_(response, behavior, message_template)
}

http426_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPUpgradeRequired$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http426 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http426")
}
#' @export
http426.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http426' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http426.response <- http426_
#' @export
http426.HttpResponse <- http426_
#' @export
http426.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http426_(response, behavior, message_template)
}

http428_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPPreconditionRequired$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http428 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http428")
}
#' @export
http428.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http428' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http428.response <- http428_
#' @export
http428.HttpResponse <- http428_
#' @export
http428.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http428_(response, behavior, message_template)
}

http429_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPTooManyRequests$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http429 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http429")
}
#' @export
http429.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http429' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http429.response <- http429_
#' @export
http429.HttpResponse <- http429_
#' @export
http429.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http429_(response, behavior, message_template)
}

http431_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPRequestHeaderFieldsTooLarge$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http431 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http431")
}
#' @export
http431.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http431' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http431.response <- http431_
#' @export
http431.HttpResponse <- http431_
#' @export
http431.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http431_(response, behavior, message_template)
}

http440_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPLoginTimeout$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http440 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http440")
}
#' @export
http440.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http440' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http440.response <- http440_
#' @export
http440.HttpResponse <- http440_
#' @export
http440.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http440_(response, behavior, message_template)
}

http444_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPNoResponse$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http444 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http444")
}
#' @export
http444.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http444' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http444.response <- http444_
#' @export
http444.HttpResponse <- http444_
#' @export
http444.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http444_(response, behavior, message_template)
}

http449_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPRetryWith$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http449 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http449")
}
#' @export
http449.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http449' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http449.response <- http449_
#' @export
http449.HttpResponse <- http449_
#' @export
http449.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http449_(response, behavior, message_template)
}

http450_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPBlockedByWindowsParentalControls$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http450 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http450")
}
#' @export
http450.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http450' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http450.response <- http450_
#' @export
http450.HttpResponse <- http450_
#' @export
http450.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http450_(response, behavior, message_template)
}

http451_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPUnavailableForLegalReasons$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http451 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http451")
}
#' @export
http451.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http451' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http451.response <- http451_
#' @export
http451.HttpResponse <- http451_
#' @export
http451.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http451_(response, behavior, message_template)
}

http494_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPRequestHeaderTooLarge$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http494 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http494")
}
#' @export
http494.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http494' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http494.response <- http494_
#' @export
http494.HttpResponse <- http494_
#' @export
http494.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http494_(response, behavior, message_template)
}

http495_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPCertError$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http495 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http495")
}
#' @export
http495.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http495' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http495.response <- http495_
#' @export
http495.HttpResponse <- http495_
#' @export
http495.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http495_(response, behavior, message_template)
}

http496_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPNoCert$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http496 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http496")
}
#' @export
http496.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http496' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http496.response <- http496_
#' @export
http496.HttpResponse <- http496_
#' @export
http496.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http496_(response, behavior, message_template)
}

http497_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPHTTPToHTTPS$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http497 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http497")
}
#' @export
http497.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http497' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http497.response <- http497_
#' @export
http497.HttpResponse <- http497_
#' @export
http497.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http497_(response, behavior, message_template)
}

http498_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPTokenExpiredInvalid$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http498 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http498")
}
#' @export
http498.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http498' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http498.response <- http498_
#' @export
http498.HttpResponse <- http498_
#' @export
http498.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http498_(response, behavior, message_template)
}

http499_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPClientClosedRequest$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http499 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http499")
}
#' @export
http499.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http499' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http499.response <- http499_
#' @export
http499.HttpResponse <- http499_
#' @export
http499.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http499_(response, behavior, message_template)
}

http500_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPInternalServerError$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http500 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http500")
}
#' @export
http500.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http500' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http500.response <- http500_
#' @export
http500.HttpResponse <- http500_
#' @export
http500.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http500_(response, behavior, message_template)
}

http501_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPNotImplemented$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http501 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http501")
}
#' @export
http501.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http501' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http501.response <- http501_
#' @export
http501.HttpResponse <- http501_
#' @export
http501.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http501_(response, behavior, message_template)
}

http502_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPBadGateway$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http502 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http502")
}
#' @export
http502.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http502' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http502.response <- http502_
#' @export
http502.HttpResponse <- http502_
#' @export
http502.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http502_(response, behavior, message_template)
}

http503_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPServiceUnavailable$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http503 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http503")
}
#' @export
http503.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http503' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http503.response <- http503_
#' @export
http503.HttpResponse <- http503_
#' @export
http503.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http503_(response, behavior, message_template)
}

http504_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPGatewayTimeout$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http504 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http504")
}
#' @export
http504.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http504' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http504.response <- http504_
#' @export
http504.HttpResponse <- http504_
#' @export
http504.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http504_(response, behavior, message_template)
}

http505_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPHTTPVersionNotSupported$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http505 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http505")
}
#' @export
http505.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http505' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http505.response <- http505_
#' @export
http505.HttpResponse <- http505_
#' @export
http505.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http505_(response, behavior, message_template)
}

http506_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPVariantAlsoNegotiates$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http506 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http506")
}
#' @export
http506.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http506' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http506.response <- http506_
#' @export
http506.HttpResponse <- http506_
#' @export
http506.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http506_(response, behavior, message_template)
}

http507_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPInsufficientStorage$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http507 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http507")
}
#' @export
http507.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http507' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http507.response <- http507_
#' @export
http507.HttpResponse <- http507_
#' @export
http507.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http507_(response, behavior, message_template)
}

http508_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPLoopDetected$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http508 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http508")
}
#' @export
http508.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http508' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http508.response <- http508_
#' @export
http508.HttpResponse <- http508_
#' @export
http508.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http508_(response, behavior, message_template)
}

http509_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPBandwidthLimitExceeded$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http509 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http509")
}
#' @export
http509.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http509' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http509.response <- http509_
#' @export
http509.HttpResponse <- http509_
#' @export
http509.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http509_(response, behavior, message_template)
}

http510_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPNotExtended$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http510 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http510")
}
#' @export
http510.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http510' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http510.response <- http510_
#' @export
http510.HttpResponse <- http510_
#' @export
http510.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http510_(response, behavior, message_template)
}

http511_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPNetworkAuthenticationRequired$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http511 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http511")
}
#' @export
http511.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http511' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http511.response <- http511_
#' @export
http511.HttpResponse <- http511_
#' @export
http511.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http511_(response, behavior, message_template)
}

http598_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPNetworkReadTimeoutError$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http598 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http598")
}
#' @export
http598.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http598' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http598.response <- http598_
#' @export
http598.HttpResponse <- http598_
#' @export
http598.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http598_(response, behavior, message_template)
}

http599_ <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stat <- fetch_status(response)
  if (muffle) if (stat < 300) return(invisible(response))
  behavior <- toggle_behavior(stat, behavior)
  tmp <- HTTPNetworkConnectTimeoutError$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
#' @rdname http
http599 <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  UseMethod("http599")
}
#' @export
http599.default <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  stop("no 'http599' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http599.response <- http599_
#' @export
http599.HttpResponse <- http599_
#' @export
http599.list <- function(response, behavior = "auto", message_template, muffle = FALSE) {
  check_curl_list(response)
  http599_(response, behavior, message_template)
}

# generated from extra/generate.R
context("http* star functions")

library("crul")
load("cached_teapot.rda"); res <- cached_teapot


test_that("http100 works", {
  skip_on_cran()

  expect_message(http100(res, behavior = "message"), "I'm a teapot")
  expect_warning(http100(res, behavior = "warning"), "I'm a teapot")
  expect_error(http100(res))

  expect_null(suppressMessages(http100(res, behavior = "message")))
})

test_that("http100 fails well", {
  skip_on_cran()

  expect_error(http100(5), "no 'http100' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http100(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http100(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http100 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http100(list()),
     "curl response list must have named elements")
  expect_error(http100(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http100(alst), "is not TRUE")
})


test_that("http101 works", {
  skip_on_cran()

  expect_message(http101(res, behavior = "message"), "I'm a teapot")
  expect_warning(http101(res, behavior = "warning"), "I'm a teapot")
  expect_error(http101(res))

  expect_null(suppressMessages(http101(res, behavior = "message")))
})

test_that("http101 fails well", {
  skip_on_cran()

  expect_error(http101(5), "no 'http101' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http101(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http101(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http101 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http101(list()),
     "curl response list must have named elements")
  expect_error(http101(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http101(alst), "is not TRUE")
})


test_that("http102 works", {
  skip_on_cran()

  expect_message(http102(res, behavior = "message"), "I'm a teapot")
  expect_warning(http102(res, behavior = "warning"), "I'm a teapot")
  expect_error(http102(res))

  expect_null(suppressMessages(http102(res, behavior = "message")))
})

test_that("http102 fails well", {
  skip_on_cran()

  expect_error(http102(5), "no 'http102' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http102(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http102(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http102 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http102(list()),
     "curl response list must have named elements")
  expect_error(http102(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http102(alst), "is not TRUE")
})


test_that("http200 works", {
  skip_on_cran()

  expect_message(http200(res, behavior = "message"), "I'm a teapot")
  expect_warning(http200(res, behavior = "warning"), "I'm a teapot")
  expect_error(http200(res))

  expect_null(suppressMessages(http200(res, behavior = "message")))
})

test_that("http200 fails well", {
  skip_on_cran()

  expect_error(http200(5), "no 'http200' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http200(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http200(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http200 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http200(list()),
     "curl response list must have named elements")
  expect_error(http200(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http200(alst), "is not TRUE")
})


test_that("http201 works", {
  skip_on_cran()

  expect_message(http201(res, behavior = "message"), "I'm a teapot")
  expect_warning(http201(res, behavior = "warning"), "I'm a teapot")
  expect_error(http201(res))

  expect_null(suppressMessages(http201(res, behavior = "message")))
})

test_that("http201 fails well", {
  skip_on_cran()

  expect_error(http201(5), "no 'http201' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http201(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http201(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http201 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http201(list()),
     "curl response list must have named elements")
  expect_error(http201(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http201(alst), "is not TRUE")
})


test_that("http202 works", {
  skip_on_cran()

  expect_message(http202(res, behavior = "message"), "I'm a teapot")
  expect_warning(http202(res, behavior = "warning"), "I'm a teapot")
  expect_error(http202(res))

  expect_null(suppressMessages(http202(res, behavior = "message")))
})

test_that("http202 fails well", {
  skip_on_cran()

  expect_error(http202(5), "no 'http202' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http202(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http202(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http202 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http202(list()),
     "curl response list must have named elements")
  expect_error(http202(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http202(alst), "is not TRUE")
})


test_that("http203 works", {
  skip_on_cran()

  expect_message(http203(res, behavior = "message"), "I'm a teapot")
  expect_warning(http203(res, behavior = "warning"), "I'm a teapot")
  expect_error(http203(res))

  expect_null(suppressMessages(http203(res, behavior = "message")))
})

test_that("http203 fails well", {
  skip_on_cran()

  expect_error(http203(5), "no 'http203' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http203(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http203(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http203 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http203(list()),
     "curl response list must have named elements")
  expect_error(http203(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http203(alst), "is not TRUE")
})


test_that("http204 works", {
  skip_on_cran()

  expect_message(http204(res, behavior = "message"), "I'm a teapot")
  expect_warning(http204(res, behavior = "warning"), "I'm a teapot")
  expect_error(http204(res))

  expect_null(suppressMessages(http204(res, behavior = "message")))
})

test_that("http204 fails well", {
  skip_on_cran()

  expect_error(http204(5), "no 'http204' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http204(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http204(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http204 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http204(list()),
     "curl response list must have named elements")
  expect_error(http204(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http204(alst), "is not TRUE")
})


test_that("http205 works", {
  skip_on_cran()

  expect_message(http205(res, behavior = "message"), "I'm a teapot")
  expect_warning(http205(res, behavior = "warning"), "I'm a teapot")
  expect_error(http205(res))

  expect_null(suppressMessages(http205(res, behavior = "message")))
})

test_that("http205 fails well", {
  skip_on_cran()

  expect_error(http205(5), "no 'http205' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http205(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http205(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http205 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http205(list()),
     "curl response list must have named elements")
  expect_error(http205(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http205(alst), "is not TRUE")
})


test_that("http206 works", {
  skip_on_cran()

  expect_message(http206(res, behavior = "message"), "I'm a teapot")
  expect_warning(http206(res, behavior = "warning"), "I'm a teapot")
  expect_error(http206(res))

  expect_null(suppressMessages(http206(res, behavior = "message")))
})

test_that("http206 fails well", {
  skip_on_cran()

  expect_error(http206(5), "no 'http206' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http206(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http206(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http206 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http206(list()),
     "curl response list must have named elements")
  expect_error(http206(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http206(alst), "is not TRUE")
})


test_that("http207 works", {
  skip_on_cran()

  expect_message(http207(res, behavior = "message"), "I'm a teapot")
  expect_warning(http207(res, behavior = "warning"), "I'm a teapot")
  expect_error(http207(res))

  expect_null(suppressMessages(http207(res, behavior = "message")))
})

test_that("http207 fails well", {
  skip_on_cran()

  expect_error(http207(5), "no 'http207' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http207(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http207(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http207 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http207(list()),
     "curl response list must have named elements")
  expect_error(http207(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http207(alst), "is not TRUE")
})


test_that("http208 works", {
  skip_on_cran()

  expect_message(http208(res, behavior = "message"), "I'm a teapot")
  expect_warning(http208(res, behavior = "warning"), "I'm a teapot")
  expect_error(http208(res))

  expect_null(suppressMessages(http208(res, behavior = "message")))
})

test_that("http208 fails well", {
  skip_on_cran()

  expect_error(http208(5), "no 'http208' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http208(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http208(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http208 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http208(list()),
     "curl response list must have named elements")
  expect_error(http208(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http208(alst), "is not TRUE")
})


test_that("http226 works", {
  skip_on_cran()

  expect_message(http226(res, behavior = "message"), "I'm a teapot")
  expect_warning(http226(res, behavior = "warning"), "I'm a teapot")
  expect_error(http226(res))

  expect_null(suppressMessages(http226(res, behavior = "message")))
})

test_that("http226 fails well", {
  skip_on_cran()

  expect_error(http226(5), "no 'http226' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http226(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http226(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http226 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http226(list()),
     "curl response list must have named elements")
  expect_error(http226(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http226(alst), "is not TRUE")
})


test_that("http300 works", {
  skip_on_cran()

  expect_message(http300(res, behavior = "message"), "I'm a teapot")
  expect_warning(http300(res, behavior = "warning"), "I'm a teapot")
  expect_error(http300(res))

  expect_null(suppressMessages(http300(res, behavior = "message")))
})

test_that("http300 fails well", {
  skip_on_cran()

  expect_error(http300(5), "no 'http300' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http300(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http300(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http300 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http300(list()),
     "curl response list must have named elements")
  expect_error(http300(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http300(alst), "is not TRUE")
})


test_that("http301 works", {
  skip_on_cran()

  expect_message(http301(res, behavior = "message"), "I'm a teapot")
  expect_warning(http301(res, behavior = "warning"), "I'm a teapot")
  expect_error(http301(res))

  expect_null(suppressMessages(http301(res, behavior = "message")))
})

test_that("http301 fails well", {
  skip_on_cran()

  expect_error(http301(5), "no 'http301' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http301(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http301(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http301 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http301(list()),
     "curl response list must have named elements")
  expect_error(http301(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http301(alst), "is not TRUE")
})


test_that("http302 works", {
  skip_on_cran()

  expect_message(http302(res, behavior = "message"), "I'm a teapot")
  expect_warning(http302(res, behavior = "warning"), "I'm a teapot")
  expect_error(http302(res))

  expect_null(suppressMessages(http302(res, behavior = "message")))
})

test_that("http302 fails well", {
  skip_on_cran()

  expect_error(http302(5), "no 'http302' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http302(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http302(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http302 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http302(list()),
     "curl response list must have named elements")
  expect_error(http302(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http302(alst), "is not TRUE")
})


test_that("http303 works", {
  skip_on_cran()

  expect_message(http303(res, behavior = "message"), "I'm a teapot")
  expect_warning(http303(res, behavior = "warning"), "I'm a teapot")
  expect_error(http303(res))

  expect_null(suppressMessages(http303(res, behavior = "message")))
})

test_that("http303 fails well", {
  skip_on_cran()

  expect_error(http303(5), "no 'http303' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http303(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http303(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http303 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http303(list()),
     "curl response list must have named elements")
  expect_error(http303(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http303(alst), "is not TRUE")
})


test_that("http304 works", {
  skip_on_cran()

  expect_message(http304(res, behavior = "message"), "I'm a teapot")
  expect_warning(http304(res, behavior = "warning"), "I'm a teapot")
  expect_error(http304(res))

  expect_null(suppressMessages(http304(res, behavior = "message")))
})

test_that("http304 fails well", {
  skip_on_cran()

  expect_error(http304(5), "no 'http304' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http304(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http304(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http304 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http304(list()),
     "curl response list must have named elements")
  expect_error(http304(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http304(alst), "is not TRUE")
})


test_that("http305 works", {
  skip_on_cran()

  expect_message(http305(res, behavior = "message"), "I'm a teapot")
  expect_warning(http305(res, behavior = "warning"), "I'm a teapot")
  expect_error(http305(res))

  expect_null(suppressMessages(http305(res, behavior = "message")))
})

test_that("http305 fails well", {
  skip_on_cran()

  expect_error(http305(5), "no 'http305' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http305(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http305(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http305 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http305(list()),
     "curl response list must have named elements")
  expect_error(http305(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http305(alst), "is not TRUE")
})


test_that("http306 works", {
  skip_on_cran()

  expect_message(http306(res, behavior = "message"), "I'm a teapot")
  expect_warning(http306(res, behavior = "warning"), "I'm a teapot")
  expect_error(http306(res))

  expect_null(suppressMessages(http306(res, behavior = "message")))
})

test_that("http306 fails well", {
  skip_on_cran()

  expect_error(http306(5), "no 'http306' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http306(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http306(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http306 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http306(list()),
     "curl response list must have named elements")
  expect_error(http306(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http306(alst), "is not TRUE")
})


test_that("http307 works", {
  skip_on_cran()

  expect_message(http307(res, behavior = "message"), "I'm a teapot")
  expect_warning(http307(res, behavior = "warning"), "I'm a teapot")
  expect_error(http307(res))

  expect_null(suppressMessages(http307(res, behavior = "message")))
})

test_that("http307 fails well", {
  skip_on_cran()

  expect_error(http307(5), "no 'http307' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http307(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http307(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http307 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http307(list()),
     "curl response list must have named elements")
  expect_error(http307(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http307(alst), "is not TRUE")
})


test_that("http308 works", {
  skip_on_cran()

  expect_message(http308(res, behavior = "message"), "I'm a teapot")
  expect_warning(http308(res, behavior = "warning"), "I'm a teapot")
  expect_error(http308(res))

  expect_null(suppressMessages(http308(res, behavior = "message")))
})

test_that("http308 fails well", {
  skip_on_cran()

  expect_error(http308(5), "no 'http308' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http308(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http308(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http308 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http308(list()),
     "curl response list must have named elements")
  expect_error(http308(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http308(alst), "is not TRUE")
})


test_that("http400 works", {
  skip_on_cran()

  expect_message(http400(res, behavior = "message"), "I'm a teapot")
  expect_warning(http400(res, behavior = "warning"), "I'm a teapot")
  expect_error(http400(res))

  expect_null(suppressMessages(http400(res, behavior = "message")))
})

test_that("http400 fails well", {
  skip_on_cran()

  expect_error(http400(5), "no 'http400' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http400(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http400(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http400 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http400(list()),
     "curl response list must have named elements")
  expect_error(http400(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http400(alst), "is not TRUE")
})


test_that("http401 works", {
  skip_on_cran()

  expect_message(http401(res, behavior = "message"), "I'm a teapot")
  expect_warning(http401(res, behavior = "warning"), "I'm a teapot")
  expect_error(http401(res))

  expect_null(suppressMessages(http401(res, behavior = "message")))
})

test_that("http401 fails well", {
  skip_on_cran()

  expect_error(http401(5), "no 'http401' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http401(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http401(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http401 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http401(list()),
     "curl response list must have named elements")
  expect_error(http401(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http401(alst), "is not TRUE")
})


test_that("http402 works", {
  skip_on_cran()

  expect_message(http402(res, behavior = "message"), "I'm a teapot")
  expect_warning(http402(res, behavior = "warning"), "I'm a teapot")
  expect_error(http402(res))

  expect_null(suppressMessages(http402(res, behavior = "message")))
})

test_that("http402 fails well", {
  skip_on_cran()

  expect_error(http402(5), "no 'http402' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http402(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http402(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http402 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http402(list()),
     "curl response list must have named elements")
  expect_error(http402(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http402(alst), "is not TRUE")
})


test_that("http403 works", {
  skip_on_cran()

  expect_message(http403(res, behavior = "message"), "I'm a teapot")
  expect_warning(http403(res, behavior = "warning"), "I'm a teapot")
  expect_error(http403(res))

  expect_null(suppressMessages(http403(res, behavior = "message")))
})

test_that("http403 fails well", {
  skip_on_cran()

  expect_error(http403(5), "no 'http403' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http403(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http403(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http403 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http403(list()),
     "curl response list must have named elements")
  expect_error(http403(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http403(alst), "is not TRUE")
})


test_that("http404 works", {
  skip_on_cran()

  expect_message(http404(res, behavior = "message"), "I'm a teapot")
  expect_warning(http404(res, behavior = "warning"), "I'm a teapot")
  expect_error(http404(res))

  expect_null(suppressMessages(http404(res, behavior = "message")))
})

test_that("http404 fails well", {
  skip_on_cran()

  expect_error(http404(5), "no 'http404' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http404(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http404(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http404 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http404(list()),
     "curl response list must have named elements")
  expect_error(http404(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http404(alst), "is not TRUE")
})


test_that("http405 works", {
  skip_on_cran()

  expect_message(http405(res, behavior = "message"), "I'm a teapot")
  expect_warning(http405(res, behavior = "warning"), "I'm a teapot")
  expect_error(http405(res))

  expect_null(suppressMessages(http405(res, behavior = "message")))
})

test_that("http405 fails well", {
  skip_on_cran()

  expect_error(http405(5), "no 'http405' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http405(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http405(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http405 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http405(list()),
     "curl response list must have named elements")
  expect_error(http405(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http405(alst), "is not TRUE")
})


test_that("http406 works", {
  skip_on_cran()

  expect_message(http406(res, behavior = "message"), "I'm a teapot")
  expect_warning(http406(res, behavior = "warning"), "I'm a teapot")
  expect_error(http406(res))

  expect_null(suppressMessages(http406(res, behavior = "message")))
})

test_that("http406 fails well", {
  skip_on_cran()

  expect_error(http406(5), "no 'http406' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http406(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http406(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http406 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http406(list()),
     "curl response list must have named elements")
  expect_error(http406(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http406(alst), "is not TRUE")
})


test_that("http407 works", {
  skip_on_cran()

  expect_message(http407(res, behavior = "message"), "I'm a teapot")
  expect_warning(http407(res, behavior = "warning"), "I'm a teapot")
  expect_error(http407(res))

  expect_null(suppressMessages(http407(res, behavior = "message")))
})

test_that("http407 fails well", {
  skip_on_cran()

  expect_error(http407(5), "no 'http407' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http407(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http407(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http407 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http407(list()),
     "curl response list must have named elements")
  expect_error(http407(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http407(alst), "is not TRUE")
})


test_that("http408 works", {
  skip_on_cran()

  expect_message(http408(res, behavior = "message"), "I'm a teapot")
  expect_warning(http408(res, behavior = "warning"), "I'm a teapot")
  expect_error(http408(res))

  expect_null(suppressMessages(http408(res, behavior = "message")))
})

test_that("http408 fails well", {
  skip_on_cran()

  expect_error(http408(5), "no 'http408' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http408(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http408(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http408 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http408(list()),
     "curl response list must have named elements")
  expect_error(http408(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http408(alst), "is not TRUE")
})


test_that("http409 works", {
  skip_on_cran()

  expect_message(http409(res, behavior = "message"), "I'm a teapot")
  expect_warning(http409(res, behavior = "warning"), "I'm a teapot")
  expect_error(http409(res))

  expect_null(suppressMessages(http409(res, behavior = "message")))
})

test_that("http409 fails well", {
  skip_on_cran()

  expect_error(http409(5), "no 'http409' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http409(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http409(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http409 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http409(list()),
     "curl response list must have named elements")
  expect_error(http409(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http409(alst), "is not TRUE")
})


test_that("http410 works", {
  skip_on_cran()

  expect_message(http410(res, behavior = "message"), "I'm a teapot")
  expect_warning(http410(res, behavior = "warning"), "I'm a teapot")
  expect_error(http410(res))

  expect_null(suppressMessages(http410(res, behavior = "message")))
})

test_that("http410 fails well", {
  skip_on_cran()

  expect_error(http410(5), "no 'http410' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http410(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http410(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http410 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http410(list()),
     "curl response list must have named elements")
  expect_error(http410(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http410(alst), "is not TRUE")
})


test_that("http411 works", {
  skip_on_cran()

  expect_message(http411(res, behavior = "message"), "I'm a teapot")
  expect_warning(http411(res, behavior = "warning"), "I'm a teapot")
  expect_error(http411(res))

  expect_null(suppressMessages(http411(res, behavior = "message")))
})

test_that("http411 fails well", {
  skip_on_cran()

  expect_error(http411(5), "no 'http411' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http411(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http411(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http411 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http411(list()),
     "curl response list must have named elements")
  expect_error(http411(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http411(alst), "is not TRUE")
})


test_that("http412 works", {
  skip_on_cran()

  expect_message(http412(res, behavior = "message"), "I'm a teapot")
  expect_warning(http412(res, behavior = "warning"), "I'm a teapot")
  expect_error(http412(res))

  expect_null(suppressMessages(http412(res, behavior = "message")))
})

test_that("http412 fails well", {
  skip_on_cran()

  expect_error(http412(5), "no 'http412' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http412(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http412(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http412 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http412(list()),
     "curl response list must have named elements")
  expect_error(http412(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http412(alst), "is not TRUE")
})


test_that("http413 works", {
  skip_on_cran()

  expect_message(http413(res, behavior = "message"), "I'm a teapot")
  expect_warning(http413(res, behavior = "warning"), "I'm a teapot")
  expect_error(http413(res))

  expect_null(suppressMessages(http413(res, behavior = "message")))
})

test_that("http413 fails well", {
  skip_on_cran()

  expect_error(http413(5), "no 'http413' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http413(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http413(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http413 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http413(list()),
     "curl response list must have named elements")
  expect_error(http413(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http413(alst), "is not TRUE")
})


test_that("http414 works", {
  skip_on_cran()

  expect_message(http414(res, behavior = "message"), "I'm a teapot")
  expect_warning(http414(res, behavior = "warning"), "I'm a teapot")
  expect_error(http414(res))

  expect_null(suppressMessages(http414(res, behavior = "message")))
})

test_that("http414 fails well", {
  skip_on_cran()

  expect_error(http414(5), "no 'http414' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http414(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http414(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http414 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http414(list()),
     "curl response list must have named elements")
  expect_error(http414(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http414(alst), "is not TRUE")
})


test_that("http415 works", {
  skip_on_cran()

  expect_message(http415(res, behavior = "message"), "I'm a teapot")
  expect_warning(http415(res, behavior = "warning"), "I'm a teapot")
  expect_error(http415(res))

  expect_null(suppressMessages(http415(res, behavior = "message")))
})

test_that("http415 fails well", {
  skip_on_cran()

  expect_error(http415(5), "no 'http415' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http415(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http415(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http415 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http415(list()),
     "curl response list must have named elements")
  expect_error(http415(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http415(alst), "is not TRUE")
})


test_that("http416 works", {
  skip_on_cran()

  expect_message(http416(res, behavior = "message"), "I'm a teapot")
  expect_warning(http416(res, behavior = "warning"), "I'm a teapot")
  expect_error(http416(res))

  expect_null(suppressMessages(http416(res, behavior = "message")))
})

test_that("http416 fails well", {
  skip_on_cran()

  expect_error(http416(5), "no 'http416' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http416(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http416(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http416 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http416(list()),
     "curl response list must have named elements")
  expect_error(http416(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http416(alst), "is not TRUE")
})


test_that("http417 works", {
  skip_on_cran()

  expect_message(http417(res, behavior = "message"), "I'm a teapot")
  expect_warning(http417(res, behavior = "warning"), "I'm a teapot")
  expect_error(http417(res))

  expect_null(suppressMessages(http417(res, behavior = "message")))
})

test_that("http417 fails well", {
  skip_on_cran()

  expect_error(http417(5), "no 'http417' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http417(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http417(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http417 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http417(list()),
     "curl response list must have named elements")
  expect_error(http417(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http417(alst), "is not TRUE")
})


test_that("http418 works", {
  skip_on_cran()

  expect_message(http418(res, behavior = "message"), "I'm a teapot")
  expect_warning(http418(res, behavior = "warning"), "I'm a teapot")
  expect_error(http418(res))

  expect_null(suppressMessages(http418(res, behavior = "message")))
})

test_that("http418 fails well", {
  skip_on_cran()

  expect_error(http418(5), "no 'http418' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http418(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http418(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http418 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http418(list()),
     "curl response list must have named elements")
  expect_error(http418(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http418(alst), "is not TRUE")
})


test_that("http419 works", {
  skip_on_cran()

  expect_message(http419(res, behavior = "message"), "I'm a teapot")
  expect_warning(http419(res, behavior = "warning"), "I'm a teapot")
  expect_error(http419(res))

  expect_null(suppressMessages(http419(res, behavior = "message")))
})

test_that("http419 fails well", {
  skip_on_cran()

  expect_error(http419(5), "no 'http419' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http419(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http419(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http419 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http419(list()),
     "curl response list must have named elements")
  expect_error(http419(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http419(alst), "is not TRUE")
})


test_that("http420 works", {
  skip_on_cran()

  expect_message(http420(res, behavior = "message"), "I'm a teapot")
  expect_warning(http420(res, behavior = "warning"), "I'm a teapot")
  expect_error(http420(res))

  expect_null(suppressMessages(http420(res, behavior = "message")))
})

test_that("http420 fails well", {
  skip_on_cran()

  expect_error(http420(5), "no 'http420' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http420(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http420(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http420 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http420(list()),
     "curl response list must have named elements")
  expect_error(http420(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http420(alst), "is not TRUE")
})


test_that("http422 works", {
  skip_on_cran()

  expect_message(http422(res, behavior = "message"), "I'm a teapot")
  expect_warning(http422(res, behavior = "warning"), "I'm a teapot")
  expect_error(http422(res))

  expect_null(suppressMessages(http422(res, behavior = "message")))
})

test_that("http422 fails well", {
  skip_on_cran()

  expect_error(http422(5), "no 'http422' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http422(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http422(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http422 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http422(list()),
     "curl response list must have named elements")
  expect_error(http422(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http422(alst), "is not TRUE")
})


test_that("http423 works", {
  skip_on_cran()

  expect_message(http423(res, behavior = "message"), "I'm a teapot")
  expect_warning(http423(res, behavior = "warning"), "I'm a teapot")
  expect_error(http423(res))

  expect_null(suppressMessages(http423(res, behavior = "message")))
})

test_that("http423 fails well", {
  skip_on_cran()

  expect_error(http423(5), "no 'http423' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http423(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http423(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http423 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http423(list()),
     "curl response list must have named elements")
  expect_error(http423(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http423(alst), "is not TRUE")
})


test_that("http424 works", {
  skip_on_cran()

  expect_message(http424(res, behavior = "message"), "I'm a teapot")
  expect_warning(http424(res, behavior = "warning"), "I'm a teapot")
  expect_error(http424(res))

  expect_null(suppressMessages(http424(res, behavior = "message")))
})

test_that("http424 fails well", {
  skip_on_cran()

  expect_error(http424(5), "no 'http424' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http424(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http424(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http424 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http424(list()),
     "curl response list must have named elements")
  expect_error(http424(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http424(alst), "is not TRUE")
})


test_that("http425 works", {
  skip_on_cran()

  expect_message(http425(res, behavior = "message"), "I'm a teapot")
  expect_warning(http425(res, behavior = "warning"), "I'm a teapot")
  expect_error(http425(res))

  expect_null(suppressMessages(http425(res, behavior = "message")))
})

test_that("http425 fails well", {
  skip_on_cran()

  expect_error(http425(5), "no 'http425' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http425(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http425(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http425 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http425(list()),
     "curl response list must have named elements")
  expect_error(http425(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http425(alst), "is not TRUE")
})


test_that("http426 works", {
  skip_on_cran()

  expect_message(http426(res, behavior = "message"), "I'm a teapot")
  expect_warning(http426(res, behavior = "warning"), "I'm a teapot")
  expect_error(http426(res))

  expect_null(suppressMessages(http426(res, behavior = "message")))
})

test_that("http426 fails well", {
  skip_on_cran()

  expect_error(http426(5), "no 'http426' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http426(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http426(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http426 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http426(list()),
     "curl response list must have named elements")
  expect_error(http426(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http426(alst), "is not TRUE")
})


test_that("http428 works", {
  skip_on_cran()

  expect_message(http428(res, behavior = "message"), "I'm a teapot")
  expect_warning(http428(res, behavior = "warning"), "I'm a teapot")
  expect_error(http428(res))

  expect_null(suppressMessages(http428(res, behavior = "message")))
})

test_that("http428 fails well", {
  skip_on_cran()

  expect_error(http428(5), "no 'http428' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http428(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http428(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http428 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http428(list()),
     "curl response list must have named elements")
  expect_error(http428(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http428(alst), "is not TRUE")
})


test_that("http429 works", {
  skip_on_cran()

  expect_message(http429(res, behavior = "message"), "I'm a teapot")
  expect_warning(http429(res, behavior = "warning"), "I'm a teapot")
  expect_error(http429(res))

  expect_null(suppressMessages(http429(res, behavior = "message")))
})

test_that("http429 fails well", {
  skip_on_cran()

  expect_error(http429(5), "no 'http429' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http429(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http429(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http429 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http429(list()),
     "curl response list must have named elements")
  expect_error(http429(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http429(alst), "is not TRUE")
})


test_that("http431 works", {
  skip_on_cran()

  expect_message(http431(res, behavior = "message"), "I'm a teapot")
  expect_warning(http431(res, behavior = "warning"), "I'm a teapot")
  expect_error(http431(res))

  expect_null(suppressMessages(http431(res, behavior = "message")))
})

test_that("http431 fails well", {
  skip_on_cran()

  expect_error(http431(5), "no 'http431' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http431(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http431(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http431 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http431(list()),
     "curl response list must have named elements")
  expect_error(http431(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http431(alst), "is not TRUE")
})


test_that("http440 works", {
  skip_on_cran()

  expect_message(http440(res, behavior = "message"), "I'm a teapot")
  expect_warning(http440(res, behavior = "warning"), "I'm a teapot")
  expect_error(http440(res))

  expect_null(suppressMessages(http440(res, behavior = "message")))
})

test_that("http440 fails well", {
  skip_on_cran()

  expect_error(http440(5), "no 'http440' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http440(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http440(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http440 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http440(list()),
     "curl response list must have named elements")
  expect_error(http440(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http440(alst), "is not TRUE")
})


test_that("http444 works", {
  skip_on_cran()

  expect_message(http444(res, behavior = "message"), "I'm a teapot")
  expect_warning(http444(res, behavior = "warning"), "I'm a teapot")
  expect_error(http444(res))

  expect_null(suppressMessages(http444(res, behavior = "message")))
})

test_that("http444 fails well", {
  skip_on_cran()

  expect_error(http444(5), "no 'http444' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http444(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http444(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http444 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http444(list()),
     "curl response list must have named elements")
  expect_error(http444(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http444(alst), "is not TRUE")
})


test_that("http449 works", {
  skip_on_cran()

  expect_message(http449(res, behavior = "message"), "I'm a teapot")
  expect_warning(http449(res, behavior = "warning"), "I'm a teapot")
  expect_error(http449(res))

  expect_null(suppressMessages(http449(res, behavior = "message")))
})

test_that("http449 fails well", {
  skip_on_cran()

  expect_error(http449(5), "no 'http449' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http449(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http449(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http449 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http449(list()),
     "curl response list must have named elements")
  expect_error(http449(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http449(alst), "is not TRUE")
})


test_that("http450 works", {
  skip_on_cran()

  expect_message(http450(res, behavior = "message"), "I'm a teapot")
  expect_warning(http450(res, behavior = "warning"), "I'm a teapot")
  expect_error(http450(res))

  expect_null(suppressMessages(http450(res, behavior = "message")))
})

test_that("http450 fails well", {
  skip_on_cran()

  expect_error(http450(5), "no 'http450' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http450(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http450(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http450 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http450(list()),
     "curl response list must have named elements")
  expect_error(http450(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http450(alst), "is not TRUE")
})


test_that("http451 works", {
  skip_on_cran()

  expect_message(http451(res, behavior = "message"), "I'm a teapot")
  expect_warning(http451(res, behavior = "warning"), "I'm a teapot")
  expect_error(http451(res))

  expect_null(suppressMessages(http451(res, behavior = "message")))
})

test_that("http451 fails well", {
  skip_on_cran()

  expect_error(http451(5), "no 'http451' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http451(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http451(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http451 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http451(list()),
     "curl response list must have named elements")
  expect_error(http451(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http451(alst), "is not TRUE")
})


test_that("http494 works", {
  skip_on_cran()

  expect_message(http494(res, behavior = "message"), "I'm a teapot")
  expect_warning(http494(res, behavior = "warning"), "I'm a teapot")
  expect_error(http494(res))

  expect_null(suppressMessages(http494(res, behavior = "message")))
})

test_that("http494 fails well", {
  skip_on_cran()

  expect_error(http494(5), "no 'http494' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http494(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http494(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http494 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http494(list()),
     "curl response list must have named elements")
  expect_error(http494(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http494(alst), "is not TRUE")
})


test_that("http495 works", {
  skip_on_cran()

  expect_message(http495(res, behavior = "message"), "I'm a teapot")
  expect_warning(http495(res, behavior = "warning"), "I'm a teapot")
  expect_error(http495(res))

  expect_null(suppressMessages(http495(res, behavior = "message")))
})

test_that("http495 fails well", {
  skip_on_cran()

  expect_error(http495(5), "no 'http495' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http495(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http495(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http495 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http495(list()),
     "curl response list must have named elements")
  expect_error(http495(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http495(alst), "is not TRUE")
})


test_that("http496 works", {
  skip_on_cran()

  expect_message(http496(res, behavior = "message"), "I'm a teapot")
  expect_warning(http496(res, behavior = "warning"), "I'm a teapot")
  expect_error(http496(res))

  expect_null(suppressMessages(http496(res, behavior = "message")))
})

test_that("http496 fails well", {
  skip_on_cran()

  expect_error(http496(5), "no 'http496' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http496(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http496(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http496 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http496(list()),
     "curl response list must have named elements")
  expect_error(http496(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http496(alst), "is not TRUE")
})


test_that("http497 works", {
  skip_on_cran()

  expect_message(http497(res, behavior = "message"), "I'm a teapot")
  expect_warning(http497(res, behavior = "warning"), "I'm a teapot")
  expect_error(http497(res))

  expect_null(suppressMessages(http497(res, behavior = "message")))
})

test_that("http497 fails well", {
  skip_on_cran()

  expect_error(http497(5), "no 'http497' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http497(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http497(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http497 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http497(list()),
     "curl response list must have named elements")
  expect_error(http497(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http497(alst), "is not TRUE")
})


test_that("http498 works", {
  skip_on_cran()

  expect_message(http498(res, behavior = "message"), "I'm a teapot")
  expect_warning(http498(res, behavior = "warning"), "I'm a teapot")
  expect_error(http498(res))

  expect_null(suppressMessages(http498(res, behavior = "message")))
})

test_that("http498 fails well", {
  skip_on_cran()

  expect_error(http498(5), "no 'http498' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http498(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http498(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http498 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http498(list()),
     "curl response list must have named elements")
  expect_error(http498(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http498(alst), "is not TRUE")
})


test_that("http499 works", {
  skip_on_cran()

  expect_message(http499(res, behavior = "message"), "I'm a teapot")
  expect_warning(http499(res, behavior = "warning"), "I'm a teapot")
  expect_error(http499(res))

  expect_null(suppressMessages(http499(res, behavior = "message")))
})

test_that("http499 fails well", {
  skip_on_cran()

  expect_error(http499(5), "no 'http499' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http499(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http499(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http499 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http499(list()),
     "curl response list must have named elements")
  expect_error(http499(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http499(alst), "is not TRUE")
})


test_that("http500 works", {
  skip_on_cran()

  expect_message(http500(res, behavior = "message"), "I'm a teapot")
  expect_warning(http500(res, behavior = "warning"), "I'm a teapot")
  expect_error(http500(res))

  expect_null(suppressMessages(http500(res, behavior = "message")))
})

test_that("http500 fails well", {
  skip_on_cran()

  expect_error(http500(5), "no 'http500' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http500(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http500(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http500 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http500(list()),
     "curl response list must have named elements")
  expect_error(http500(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http500(alst), "is not TRUE")
})


test_that("http501 works", {
  skip_on_cran()

  expect_message(http501(res, behavior = "message"), "I'm a teapot")
  expect_warning(http501(res, behavior = "warning"), "I'm a teapot")
  expect_error(http501(res))

  expect_null(suppressMessages(http501(res, behavior = "message")))
})

test_that("http501 fails well", {
  skip_on_cran()

  expect_error(http501(5), "no 'http501' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http501(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http501(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http501 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http501(list()),
     "curl response list must have named elements")
  expect_error(http501(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http501(alst), "is not TRUE")
})


test_that("http502 works", {
  skip_on_cran()

  expect_message(http502(res, behavior = "message"), "I'm a teapot")
  expect_warning(http502(res, behavior = "warning"), "I'm a teapot")
  expect_error(http502(res))

  expect_null(suppressMessages(http502(res, behavior = "message")))
})

test_that("http502 fails well", {
  skip_on_cran()

  expect_error(http502(5), "no 'http502' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http502(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http502(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http502 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http502(list()),
     "curl response list must have named elements")
  expect_error(http502(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http502(alst), "is not TRUE")
})


test_that("http503 works", {
  skip_on_cran()

  expect_message(http503(res, behavior = "message"), "I'm a teapot")
  expect_warning(http503(res, behavior = "warning"), "I'm a teapot")
  expect_error(http503(res))

  expect_null(suppressMessages(http503(res, behavior = "message")))
})

test_that("http503 fails well", {
  skip_on_cran()

  expect_error(http503(5), "no 'http503' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http503(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http503(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http503 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http503(list()),
     "curl response list must have named elements")
  expect_error(http503(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http503(alst), "is not TRUE")
})


test_that("http504 works", {
  skip_on_cran()

  expect_message(http504(res, behavior = "message"), "I'm a teapot")
  expect_warning(http504(res, behavior = "warning"), "I'm a teapot")
  expect_error(http504(res))

  expect_null(suppressMessages(http504(res, behavior = "message")))
})

test_that("http504 fails well", {
  skip_on_cran()

  expect_error(http504(5), "no 'http504' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http504(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http504(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http504 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http504(list()),
     "curl response list must have named elements")
  expect_error(http504(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http504(alst), "is not TRUE")
})


test_that("http505 works", {
  skip_on_cran()

  expect_message(http505(res, behavior = "message"), "I'm a teapot")
  expect_warning(http505(res, behavior = "warning"), "I'm a teapot")
  expect_error(http505(res))

  expect_null(suppressMessages(http505(res, behavior = "message")))
})

test_that("http505 fails well", {
  skip_on_cran()

  expect_error(http505(5), "no 'http505' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http505(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http505(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http505 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http505(list()),
     "curl response list must have named elements")
  expect_error(http505(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http505(alst), "is not TRUE")
})


test_that("http506 works", {
  skip_on_cran()

  expect_message(http506(res, behavior = "message"), "I'm a teapot")
  expect_warning(http506(res, behavior = "warning"), "I'm a teapot")
  expect_error(http506(res))

  expect_null(suppressMessages(http506(res, behavior = "message")))
})

test_that("http506 fails well", {
  skip_on_cran()

  expect_error(http506(5), "no 'http506' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http506(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http506(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http506 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http506(list()),
     "curl response list must have named elements")
  expect_error(http506(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http506(alst), "is not TRUE")
})


test_that("http507 works", {
  skip_on_cran()

  expect_message(http507(res, behavior = "message"), "I'm a teapot")
  expect_warning(http507(res, behavior = "warning"), "I'm a teapot")
  expect_error(http507(res))

  expect_null(suppressMessages(http507(res, behavior = "message")))
})

test_that("http507 fails well", {
  skip_on_cran()

  expect_error(http507(5), "no 'http507' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http507(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http507(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http507 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http507(list()),
     "curl response list must have named elements")
  expect_error(http507(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http507(alst), "is not TRUE")
})


test_that("http508 works", {
  skip_on_cran()

  expect_message(http508(res, behavior = "message"), "I'm a teapot")
  expect_warning(http508(res, behavior = "warning"), "I'm a teapot")
  expect_error(http508(res))

  expect_null(suppressMessages(http508(res, behavior = "message")))
})

test_that("http508 fails well", {
  skip_on_cran()

  expect_error(http508(5), "no 'http508' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http508(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http508(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http508 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http508(list()),
     "curl response list must have named elements")
  expect_error(http508(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http508(alst), "is not TRUE")
})


test_that("http509 works", {
  skip_on_cran()

  expect_message(http509(res, behavior = "message"), "I'm a teapot")
  expect_warning(http509(res, behavior = "warning"), "I'm a teapot")
  expect_error(http509(res))

  expect_null(suppressMessages(http509(res, behavior = "message")))
})

test_that("http509 fails well", {
  skip_on_cran()

  expect_error(http509(5), "no 'http509' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http509(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http509(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http509 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http509(list()),
     "curl response list must have named elements")
  expect_error(http509(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http509(alst), "is not TRUE")
})


test_that("http510 works", {
  skip_on_cran()

  expect_message(http510(res, behavior = "message"), "I'm a teapot")
  expect_warning(http510(res, behavior = "warning"), "I'm a teapot")
  expect_error(http510(res))

  expect_null(suppressMessages(http510(res, behavior = "message")))
})

test_that("http510 fails well", {
  skip_on_cran()

  expect_error(http510(5), "no 'http510' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http510(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http510(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http510 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http510(list()),
     "curl response list must have named elements")
  expect_error(http510(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http510(alst), "is not TRUE")
})


test_that("http511 works", {
  skip_on_cran()

  expect_message(http511(res, behavior = "message"), "I'm a teapot")
  expect_warning(http511(res, behavior = "warning"), "I'm a teapot")
  expect_error(http511(res))

  expect_null(suppressMessages(http511(res, behavior = "message")))
})

test_that("http511 fails well", {
  skip_on_cran()

  expect_error(http511(5), "no 'http511' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http511(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http511(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http511 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http511(list()),
     "curl response list must have named elements")
  expect_error(http511(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http511(alst), "is not TRUE")
})


test_that("http598 works", {
  skip_on_cran()

  expect_message(http598(res, behavior = "message"), "I'm a teapot")
  expect_warning(http598(res, behavior = "warning"), "I'm a teapot")
  expect_error(http598(res))

  expect_null(suppressMessages(http598(res, behavior = "message")))
})

test_that("http598 fails well", {
  skip_on_cran()

  expect_error(http598(5), "no 'http598' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http598(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http598(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http598 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http598(list()),
     "curl response list must have named elements")
  expect_error(http598(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http598(alst), "is not TRUE")
})


test_that("http599 works", {
  skip_on_cran()

  expect_message(http599(res, behavior = "message"), "I'm a teapot")
  expect_warning(http599(res, behavior = "warning"), "I'm a teapot")
  expect_error(http599(res))

  expect_null(suppressMessages(http599(res, behavior = "message")))
})

test_that("http599 fails well", {
  skip_on_cran()

  expect_error(http599(5), "no 'http599' method for numeric")

  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http599(res, behavior = "asdfasdf"),
     "'behavior' must be one of")

  # message_template fails well
  expect_error(http599(res, message_template = 5),
     "'message_template' must be of class character")
})

test_that("http599 fails well with malformed curl pkg response objects", {
  skip_on_cran()

  expect_error(http599(list()),
     "curl response list must have named elements")
  expect_error(http599(list(a = 5)),
     "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http599(alst), "is not TRUE")
})

#' @title fauxpas
#' @description HTTP Error Helpers
#' @name fauxpas-package
#' @importFrom R6 R6Class
#' @importFrom httpcode http_code
#' @importFrom whisker whisker.render
#' @aliases fauxpas
#' @docType package
#' @author Scott Chamberlain \email{myrmecocystus@@gmail.com}
#' @keywords package
NULL

#' Find error classes
#'
#' @export
#' @param status_code (numeric,integer) A status code
#' @seealso \code{\link[fauxpas]{Error}}, \code{\link[fauxpas]{Error-Classes}}
#' @return an object of class \code{R6ClassGenerator}. call \code{$new()}
#' to initialize a new instance
#' @examples
#' find_error_class(414)
#' find_error_class(418)
#' find_error_class(505)
#'
#' # initialize the class
#' find_error_class(418)$new()
#'
#' # not found
#' \dontrun{find_error_class(999)}
find_error_class <- function(status_code) {
  codes <- vapply(fauxpas_env$http_children,
                  function(z) z$public_fields$status_code, 1)
  mtch <- which(status_code == codes)
  if (length(mtch) == 0) stop("no method found for ", status_code)
  fauxpas_env$http_children[[which(status_code == codes)]]
}

`%||%` <- function(x, y) if (is.null(x)) y else x

gv <- function(x) {
  tryCatch(
    httpcode::http_code(x, TRUE)$verbose_explanation,
    error = function(e) NULL
  ) %||%
    ""
}


error_child_generator <- function(class_name, x) {
  self <- super <- NULL
  R6::R6Class(
    class_name,
    inherit = Error,
    public = list(
      name = class_name,
      status_code = x,
      mssg = gv(x),

      do_verbose = function(
        response,
        template = self$message_template_verbose
      ) {
        super$do(response, self$mssg, template)
      }
    )
  )
}


#' Individual error classes
#'
#' These error classes are for each HTTP error, and inherit from
#' the [Error] class in this package.
#'
#' @details In addition to what's available in [Error],
#' these classes have a single variable `mssg` that is the very
#' verbose complete message describing the HTTP condition in detail.
#' You can include that message in your condition by using `do_verbose`
#' (see below)
#'
#' @section Methods:
#'
#' In addition to the methods documented in [Error], these classes
#' also have:
#'
#' * `do_verbose(response, template)`:
#'
#'   Execute condition, whether it be message, warning, or error.
#'
#'   * response: is any response from `crul`, `curl`, or `httr`
#'   Execute condition, whether it be message, warning, error, or your
#'   own custom function. This method uses `message_template_verbose`,
#'   and uses it's default value.
#'   * template: a template to use for the verbose message, see [Error]
#'   for details
#'
#' @seealso [fauxpas::Error], [fauxpas::http]
#'
#' @examples
#' HTTPRequestURITooLong$new(
#'   message_template = "{{reason}} ............ {{status}}",
#'   message_template_verbose = "{{reason}} .>.>.>.>.>.> {{status}}\n {{message}}"
#' )
#' @format NULL
#' @usage NULL
#' @name Error-Classes
NULL


# 100 ----------------------
#' @export
#' @rdname Error-Classes
HTTPContinue <- error_child_generator("HTTPContinue", 100)

#' @export
#' @rdname Error-Classes
HTTPSwitchProtocol <- error_child_generator("HTTPSwitchProtocol", 101)

#' @export
#' @rdname Error-Classes
HTTPProcessing <- error_child_generator("HTTPProcessing", 102)


# 200 ----------------------
#' @export
#' @rdname Error-Classes
HTTPOK <- error_child_generator("HTTPOK", 200)

#' @export
#' @rdname Error-Classes
HTTPCreated <- error_child_generator("HTTPCreated", 201)

#' @export
#' @rdname Error-Classes
HTTPAccepted <- error_child_generator("HTTPAccepted", 202)

#' @export
#' @rdname Error-Classes
HTTPNonAuthoritativeInformation <- error_child_generator(
  "HTTPNonAuthoritativeInformation",
  203
)

#' @export
#' @rdname Error-Classes
HTTPNoContent <- error_child_generator("HTTPNoContent", 204)

#' @export
#' @rdname Error-Classes
HTTPResetContent <- error_child_generator("HTTPResetContent", 205)

#' @export
#' @rdname Error-Classes
HTTPPartialContent <- error_child_generator("HTTPPartialContent", 206)

#' @export
#' @rdname Error-Classes
HTTPMultiStatus <- error_child_generator("HTTPMultiStatus", 207)

#' @export
#' @rdname Error-Classes
HTTPAlreadyReported <- error_child_generator("HTTPAlreadyReported", 208)

#' @export
#' @rdname Error-Classes
HTTPImUsed <- error_child_generator("HTTPImUsed", 226)


# 300 ----------------------
#' @export
#' @rdname Error-Classes
HTTPMultipleChoices <- error_child_generator("HTTPMultipleChoices", 300)

#' @export
#' @rdname Error-Classes
HTTPMovedPermanently <- error_child_generator("HTTPMovedPermanently", 301)

#' @export
#' @rdname Error-Classes
HTTPFound <- error_child_generator("HTTPFound", 302)

#' @export
#' @rdname Error-Classes
HTTPSeeOther <- error_child_generator("HTTPSeeOther", 303)

#' @export
#' @rdname Error-Classes
HTTPNotModified <- error_child_generator("HTTPNotModified", 304)

#' @export
#' @rdname Error-Classes
HTTPUseProxy <- error_child_generator("HTTPUseProxy", 305)

#' @export
#' @rdname Error-Classes
HTTPSwitchProxy <- error_child_generator("HTTPSwitchProxy", 306)

#' @export
#' @rdname Error-Classes
HTTPTemporaryRedirect <- error_child_generator("HTTPTemporaryRedirect", 307)

#' @export
#' @rdname Error-Classes
HTTPPermanentRedirect <- error_child_generator("HTTPPermanentRedirect", 308)


# 400 ----------------------
#' @export
#' @rdname Error-Classes
HTTPBadRequest <- error_child_generator("HTTPBadRequest", 400)

#' @export
#' @rdname Error-Classes
HTTPUnauthorized <- error_child_generator("HTTPUnauthorized", 401)

#' @export
#' @rdname Error-Classes
HTTPPaymentRequired <- error_child_generator("HTTPPaymentRequired", 402)

#' @export
#' @rdname Error-Classes
HTTPForbidden <- error_child_generator("HTTPForbidden", 403)

#' @export
#' @rdname Error-Classes
HTTPNotFound <- error_child_generator("HTTPNotFound", 404)

#' @export
#' @rdname Error-Classes
HTTPMethodNotAllowed <- error_child_generator("HTTPMethodNotAllowed", 405)

#' @export
#' @rdname Error-Classes
HTTPNotAcceptable <- error_child_generator("HTTPNotAcceptable", 406)

#' @export
#' @rdname Error-Classes
HTTPProxyAuthenticationRequired <- error_child_generator(
  "HTTPProxyAuthenticationRequired",
  407
)

#' @export
#' @rdname Error-Classes
HTTPRequestTimeout <- error_child_generator("HTTPRequestTimeout", 408)

#' @export
#' @rdname Error-Classes
HTTPConflict <- error_child_generator("HTTPConflict", 409)

#' @export
#' @rdname Error-Classes
HTTPGone <- error_child_generator("HTTPGone", 410)

#' @export
#' @rdname Error-Classes
HTTPLengthRequired <- error_child_generator("HTTPLengthRequired", 411)

#' @export
#' @rdname Error-Classes
HTTPPreconditionFailed <- error_child_generator("HTTPPreconditionFailed", 412)

#' @export
#' @rdname Error-Classes
HTTPRequestEntityTooLarge <- error_child_generator(
  "HTTPRequestEntityTooLarge",
  413
)

#' @export
#' @rdname Error-Classes
HTTPRequestURITooLong <- error_child_generator("HTTPRequestURITooLong", 414)

#' @export
#' @rdname Error-Classes
HTTPUnsupportedMediaType <- error_child_generator(
  "HTTPUnsupportedMediaType",
  415
)

#' @export
#' @rdname Error-Classes
HTTPRequestRangeNotSatisfiable <- error_child_generator(
  "HTTPRequestRangeNotSatisfiable",
  416
)

#' @export
#' @rdname Error-Classes
HTTPExpectationFailed <- error_child_generator("HTTPExpectationFailed", 417)

#' @export
#' @rdname Error-Classes
HTTPTeaPot <- error_child_generator("HTTPTeaPot", 418)

#' @export
#' @rdname Error-Classes
HTTPAuthenticationTimeout <- error_child_generator(
  "HTTPAuthenticationTimeout",
  419
)

#' @export
#' @rdname Error-Classes
HTTPMethodFailure <- error_child_generator("HTTPMethodFailure", 420)

#' @export
#' @rdname Error-Classes
HTTPMisdirectedRequest <- error_child_generator("HTTPMisdirectedRequest", 421)

#' @export
#' @rdname Error-Classes
HTTPUnprocessableEntity <- error_child_generator("HTTPUnprocessableEntity", 422)

#' @export
#' @rdname Error-Classes
HTTPLocked <- error_child_generator("HTTPLocked", 423)

#' @export
#' @rdname Error-Classes
HTTPFailedDependency <- error_child_generator("HTTPFailedDependency", 424)

#' @export
#' @rdname Error-Classes
HTTPUnorderedCollection <- error_child_generator("HTTPUnorderedCollection", 425)

#' @export
#' @rdname Error-Classes
HTTPUpgradeRequired <- error_child_generator("HTTPUpgradeRequired", 426)

#' @export
#' @rdname Error-Classes
HTTPPreconditionRequired <- error_child_generator(
  "HTTPPreconditionRequired",
  428
)

#' @export
#' @rdname Error-Classes
HTTPTooManyRequests <- error_child_generator("HTTPTooManyRequests", 429)

#' @export
#' @rdname Error-Classes
HTTPRequestHeaderFieldsTooLarge <- error_child_generator(
  "HTTPRequestHeaderFieldsTooLarge",
  431
)

#' @export
#' @rdname Error-Classes
HTTPLoginTimeout <- error_child_generator("HTTPLoginTimeout", 440)

#' @export
#' @rdname Error-Classes
HTTPNoResponse <- error_child_generator("HTTPNoResponse", 444)

#' @export
#' @rdname Error-Classes
HTTPRetryWith <- error_child_generator("HTTPRetryWith", 449)

#' @export
#' @rdname Error-Classes
HTTPBlockedByWindowsParentalControls <- error_child_generator(
  "HTTPBlockedByWindowsParentalControls",
  450
)

#' @export
#' @rdname Error-Classes
HTTPUnavailableForLegalReasons <- error_child_generator(
  "HTTPUnavailableForLegalReasons",
  451
)

#' @export
#' @rdname Error-Classes
HTTPRequestHeaderTooLarge <- error_child_generator(
  "HTTPRequestHeaderTooLarge",
  494
)

#' @export
#' @rdname Error-Classes
HTTPCertError <- error_child_generator("HTTPCertError", 495)

#' @export
#' @rdname Error-Classes
HTTPNoCert <- error_child_generator("HTTPNoCert", 496)

#' @export
#' @rdname Error-Classes
HTTPHTTPToHTTPS <- error_child_generator("HTTPHTTPToHTTPS", 497)

#' @export
#' @rdname Error-Classes
HTTPTokenExpiredInvalid <- error_child_generator("HTTPTokenExpiredInvalid", 498)

#' @export
#' @rdname Error-Classes
HTTPClientClosedRequest <- error_child_generator("HTTPClientClosedRequest", 499)


# 500 ----------------------

#' @export
#' @rdname Error-Classes
HTTPInternalServerError <- error_child_generator("HTTPInternalServerError", 500)

#' @export
#' @rdname Error-Classes
HTTPNotImplemented <- error_child_generator("HTTPNotImplemented", 501)

#' @export
#' @rdname Error-Classes
HTTPBadGateway <- error_child_generator("HTTPBadGateway", 502)

#' @export
#' @rdname Error-Classes
HTTPServiceUnavailable <- error_child_generator("HTTPServiceUnavailable", 503)

#' @export
#' @rdname Error-Classes
HTTPGatewayTimeout <- error_child_generator("HTTPGatewayTimeout", 504)

#' @export
#' @rdname Error-Classes
HTTPHTTPVersionNotSupported <- error_child_generator(
  "HTTPHTTPVersionNotSupported",
  505
)

#' @export
#' @rdname Error-Classes
HTTPVariantAlsoNegotiates <- error_child_generator(
  "HTTPVariantAlsoNegotiates",
  506
)

#' @export
#' @rdname Error-Classes
HTTPInsufficientStorage <- error_child_generator("HTTPInsufficientStorage", 507)

#' @export
#' @rdname Error-Classes
HTTPLoopDetected <- error_child_generator("HTTPLoopDetected", 508)

#' @export
#' @rdname Error-Classes
HTTPBandwidthLimitExceeded <- error_child_generator(
  "HTTPBandwidthLimitExceeded",
  509
)

#' @export
#' @rdname Error-Classes
HTTPNotExtended <- error_child_generator("HTTPNotExtended", 510)

#' @export
#' @rdname Error-Classes
HTTPNetworkAuthenticationRequired <- error_child_generator(
  "HTTPNetworkAuthenticationRequired",
  511
)

#' @export
#' @rdname Error-Classes
HTTPNetworkReadTimeoutError <- error_child_generator(
  "HTTPNetworkReadTimeoutError",
  598
)

#' @export
#' @rdname Error-Classes
HTTPNetworkConnectTimeoutError <- error_child_generator(
  "HTTPNetworkConnectTimeoutError",
  599
)


# Cloudflare
#' @export
#' @rdname Error-Classes
HTTPWebServerReturnedUnknownError <- error_child_generator(
  "HTTPWebServerReturnedUnknownError",
  520
)

#' @export
#' @rdname Error-Classes
HTTPWebServerIsDown <- error_child_generator("HTTPWebServerIsDown", 521)

#' @export
#' @rdname Error-Classes
HTTPConnectionTimedOut <- error_child_generator("HTTPConnectionTimedOut", 522)

#' @export
#' @rdname Error-Classes
HTTPOriginIsUnreachable <- error_child_generator("HTTPOriginIsUnreachable", 523)

#' @export
#' @rdname Error-Classes
HTTPATimeoutOccurred <- error_child_generator("HTTPATimeoutOccurred", 524)

#' @export
#' @rdname Error-Classes
HTTPSSLHandshakeFailed <- error_child_generator("HTTPSSLHandshakeFailed", 525)

#' @export
#' @rdname Error-Classes
HTTPInvalidSSLCertificate <- error_child_generator(
  "HTTPInvalidSSLCertificate ",
  526
)

#' @export
#' @rdname Error-Classes
HTTPRailgunError <- error_child_generator("HTTPRailgunError", 527)


## all children
fauxpas_env <- new.env()
assign(
  "http_children",
  list(
    HTTPContinue,
    HTTPSwitchProtocol,
    HTTPProcessing,
    HTTPOK,
    HTTPCreated,
    HTTPAccepted,
    HTTPNonAuthoritativeInformation,
    HTTPNoContent,
    HTTPResetContent,
    HTTPPartialContent,
    HTTPMultiStatus,
    HTTPAlreadyReported,
    HTTPImUsed,
    HTTPMultipleChoices,
    HTTPMovedPermanently,
    HTTPFound,
    HTTPSeeOther,
    HTTPNotModified,
    HTTPUseProxy,
    HTTPSwitchProxy,
    HTTPTemporaryRedirect,
    HTTPPermanentRedirect,
    HTTPBadRequest,
    HTTPUnauthorized,
    HTTPPaymentRequired,
    HTTPForbidden,
    HTTPNotFound,
    HTTPMethodNotAllowed,
    HTTPNotAcceptable,
    HTTPProxyAuthenticationRequired,
    HTTPRequestTimeout,
    HTTPConflict,
    HTTPGone,
    HTTPLengthRequired,
    HTTPPreconditionFailed,
    HTTPRequestEntityTooLarge,
    HTTPRequestURITooLong,
    HTTPUnsupportedMediaType,
    HTTPRequestRangeNotSatisfiable,
    HTTPExpectationFailed,
    HTTPTeaPot,
    HTTPAuthenticationTimeout,
    HTTPMethodFailure,
    HTTPMisdirectedRequest,
    HTTPUnprocessableEntity,
    HTTPLocked,
    HTTPFailedDependency,
    HTTPUnorderedCollection,
    HTTPUpgradeRequired,
    HTTPPreconditionRequired,
    HTTPTooManyRequests,
    HTTPRequestHeaderFieldsTooLarge,
    HTTPLoginTimeout,
    HTTPNoResponse,
    HTTPRetryWith,
    HTTPBlockedByWindowsParentalControls,
    HTTPUnavailableForLegalReasons,
    HTTPRequestHeaderTooLarge,
    HTTPCertError,
    HTTPNoCert,
    HTTPHTTPToHTTPS,
    HTTPTokenExpiredInvalid,
    HTTPClientClosedRequest,
    HTTPInternalServerError,
    HTTPNotImplemented,
    HTTPBadGateway,
    HTTPServiceUnavailable,
    HTTPGatewayTimeout,
    HTTPHTTPVersionNotSupported,
    HTTPVariantAlsoNegotiates,
    HTTPInsufficientStorage,
    HTTPLoopDetected,
    HTTPBandwidthLimitExceeded,
    HTTPNotExtended,
    HTTPNetworkAuthenticationRequired,
    HTTPNetworkReadTimeoutError,
    HTTPNetworkConnectTimeoutError,
    HTTPWebServerReturnedUnknownError,
    HTTPWebServerIsDown,
    HTTPConnectionTimedOut,
    HTTPOriginIsUnreachable,
    HTTPATimeoutOccurred,
    HTTPSSLHandshakeFailed,
    HTTPInvalidSSLCertificate,
    HTTPRailgunError
  ),
  envir = fauxpas_env
)

context("http children methods")

test_that("http children methods: HTTPContinue", {
  skip_on_cran()

  library("crul")
  res <- HttpClient$new("https://httpbin.org/status/405")$get()

  aa <- HTTPMethodNotAllowed$new()
  bb <- HTTPMethodNotAllowed$new(behavior = "message")
  cc <- HTTPMethodNotAllowed$new(behavior = "warning")

  expect_error(aa$do(res))
  expect_message(bb$do(res), "Method Not Allowed")
  expect_warning(cc$do(res), "Method Not Allowed")

  expect_null(suppressMessages(bb$do(res)))
  #expect_null(suppressWarnings(cc$do(res)))

  expect_error(aa$do_verbose(res))
  expect_message(bb$do_verbose(res),
                 "The method specified in the Request-Line is not allowed")
  expect_warning(cc$do_verbose(res),
                 "The method specified in the Request-Line is not allowed")
})

test_that("http fails well", {
  skip_on_cran()

  expect_error(HTTPAccepted$new(behavior = 5), "is not TRUE")
  expect_error(HTTPBadGateway$new(a = "asdfadfasf"), "unused argument")
})

#' higher level error wrappers
#'
#' @export
#' @param response The result of a call via \pkg{crul}, \pkg{curl}, or
#' \pkg{httr}
#' @param muffle (logical) whether to not respond when status codes
#' in 1xx-3xx series. Default: `FALSE`
#' @template params
#'
#' @seealso [Error], [fauxpas::Error-Classes]
#'
#' @section using package \pkg{curl}:
#' curl reponses are simple lists, so we have little to go on to make sure
#' it's a response from the \pkg{curl} package. We check for list names
#' internally but of course you could pass in a list with the right named
#' elements, while the values are complete nonsense, in which case
#' we'll probably fail badly. There's not much we can do.
#'
#' @note These `http*` methods only use `$do` and not
#' `$do_verbose`.
#'
#' @examples
#' res <- list(url="https://a.com", status_code=200, type="application/xml",
#'  headers=charToRaw("a"), modified=NA, times=5, content=charToRaw('b'))
#' http(res, behavior = "message")
http <- function(
  response,
  behavior = "auto",
  message_template,
  muffle = FALSE
) {
  UseMethod("http")
}
http_ <- function(
  response,
  behavior = "auto",
  message_template,
  muffle = FALSE
) {
  stat <- fetch_status(response)
  if (muffle) {
    if (stat < 300) return(invisible(response))
  }
  behavior <- toggle_behavior(stat, behavior)
  tmp <- Error$new(behavior = behavior, message_template = message_template)
  tmp$do(response)
}
#' @export
http.default <- function(
  response,
  behavior = "auto",
  message_template,
  muffle = FALSE
) {
  stop("no 'http' method for ", class(response)[[1L]], call. = FALSE)
}
#' @export
http.response <- http_
#' @export
http.HttpResponse <- http_
#' @export
http.list <- function(
  response,
  behavior = "auto",
  message_template,
  muffle = FALSE
) {
  check_curl_list(response)
  http_(response, behavior, message_template)
}

check_curl_list <- function(x) {
  # attempt to make sure list is from curl
  ## any names?
  if (length(names(x)) == 0) {
    stop('curl response list must have named elements - see docs')
  }
  ## the right names?
  if (
    !all(
      c('url', 'status_code', 'headers', 'modified', 'times') %in%
        names(x)
    )
  ) {
    stop(
      "found list names '",
      paste0(names(x), collapse = ", "),
      "' - indicating input not likely a curl response - see docs"
    )
  }
  ## the right classes
  stopifnot(inherits(x$url, "character"))
  stopifnot(inherits(x$headers, "raw"))
  stopifnot(inherits(x$times, "numeric"))
  ### sometimes content won't be present
  if (!is.null(x$content)) stopifnot(inherits(x$content, "raw"))
}

toggle_behavior <- function(status, behavior) {
  stopifnot(inherits(behavior, "character"))
  if (!behavior %in% c('stop', 'warning', 'message', 'auto')) {
    stop(
      "'behavior' must be one of stop, warning, message, or auto",
      call. = FALSE
    )
  }
  switch(
    behavior,
    auto = {
      status_type <- as.character((status %/% 100) * 100)
      switch(
        status_type,
        '100' = "message",
        '200' = "message",
        '300' = "warning",
        '400' = "stop",
        '500' = "stop"
      )
    },
    stop = "stop",
    warning = "warning",
    message = "message"
  )
}

context("http")

test_that("http works", {
  skip_on_cran()

  library("crul")
  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  expect_message(http(res, behavior = "message"), "I'm a teapot")
  expect_warning(http(res, behavior = "warning"), "I'm a teapot")
  expect_error(http(res))

  expect_null(suppressMessages(http(res, behavior = "message")))
  #expect_null(suppressWarnings(http(res, behavior = "warning")))
})

test_that("http fails well", {
  skip_on_cran()

  expect_error(http(5), "no 'http' method for numeric")
  expect_error(http("asdfadfasf"), "no 'http' method for character")
  expect_error(http(mtcars), "no 'http' method for data.frame")

  library("crul")
  res <- HttpClient$new("https://httpbin.org/status/418")$get()

  # behavior fails well
  expect_error(http(res, behavior = "asdfasdf"),
               "'behavior' must be one of")

  # message_template fails well
  expect_error(http(res, message_template = 5),
               "'message_template' must be of class character")
})


test_that("http fails well with malformed curl pkg response objects", {
  skip_on_cran()

  library("curl")
  expect_error(http(list()),
               "curl response list must have named elements")
  expect_error(http(list(a = 5)),
               "found list names 'a' - indicating input not likely a")
  alst <- list(url=5, status_code=5, headers=5, modified=5, times=5)
  expect_error(http(alst), "is not TRUE")
})

## ----echo=FALSE---------------------------------------------------------------
knitr::opts_chunk$set(
  comment = "#>",
  collapse = TRUE,
  warning = FALSE,
  message = FALSE
)

## ----eval=FALSE---------------------------------------------------------------
#  install.packages("fauxpas")

## ----eval=FALSE---------------------------------------------------------------
#  # install.packages("remotes")
#  remotes::install_github("sckott/fauxpas")

## -----------------------------------------------------------------------------
library("fauxpas")

## -----------------------------------------------------------------------------
(x <- find_error_class(418))

## -----------------------------------------------------------------------------
x$new()

## ----eval=FALSE---------------------------------------------------------------
#  find_error_class(999)
#  #> Error in find_error_class(999) : no method found for 999

## ----eval=FALSE---------------------------------------------------------------
#  library("crul")
#  cli <- HttpClient$new("https://httpbin.org/status/414")
#  res <- cli$get()
#  http(res)
#  #> Error: Request-URI Too Long (HTTP 414).
#  http414(res)
#  #> Error: Request-URI Too Long (HTTP 414).

## ----eval=FALSE---------------------------------------------------------------
#  x <- HTTPRequestURITooLong$new()
#  x$do_verbose(res)
#  #> Error: Request-URI Too Long (HTTP 414).
#  #> - The server is refusing to service the request because the Request-URI is
#  #>    longer than the server is willing to interpret. This rare condition is only likely
#  #>    to occur when a client has improperly converted a POST request to a GET request
#  #>    with long query information, when the client has descended into a URI black hole
#  #>    of redirection (e.g., a redirected URI prefix that points to a suffix of itself),
#  #>    or when the server is under attack by a client attempting to exploit security
#  #>    holes present in some servers using fixed-length buffers for reading or
#  #>    manipulating the Request-URI.

## ----eval=FALSE---------------------------------------------------------------
#  library("curl")
#  h <- curl::new_handle()
#  curl::handle_setopt(h)
#  resp <- curl::curl_fetch_memory("https://httpbin.org/status/404", h)
#  http(resp)
#  #> Error: Not Found (HTTP 404).
#  http404(resp)
#  #> Error: Not Found (HTTP 404).

## ----eval=FALSE---------------------------------------------------------------
#  x <- HTTPNotFound$new()
#  x$do_verbose(resp)
#  #> Error:  Not Found (HTTP 404).
#  #>  - The server has not found anything matching the Request-URI. No indication is
#  #> given of whether the condition is temporary or permanent. The 410 (Gone) status
#  #> code SHOULD be used if the server knows, through some internally configurable
#  #> mechanism, that an old resource is permanently unavailable and has no forwarding
#  #> address. #> This status code is commonly used when the server does not wish to
#  #> reveal exactly why the request has been refused, or when no other response is
#  #> applicable.

## ----eval=FALSE---------------------------------------------------------------
#  library("httr")
#  res <- GET("https://httpbin.org/status/405")
#  http405(res)
#  #> Error: Method Not Allowed (HTTP 405).

## ----eval=FALSE---------------------------------------------------------------
#  x <- HTTPMethodNotAllowed$new()
#  x$do_verbose(res)
#  #> Error: Method Not Allowed (HTTP 405).
#  #>  - The method specified in the Request-Line is not allowed for the resource
#  #> identified by the Request-URI. The response MUST include an Allow header
#  #> containing a list of valid methods for the requested resource.

#' Error class
#'
#' @export
#' @template params
#' @param call.	(logical) indicating if the call should become part
#' of the error message. Default: `FALSE`
#' @param message_template_verbose A verbose message template. optional.
#' use whisker templating. names to use are: reason, status, message.
#' use in template like `{{reason}}`, `{{status}}`, and
#' `{{message}}`. Note that this is ignored here, but is
#' used in the `HTTP*` methods (e.g. `HTTPBadRequest`)
#' @param muffle (logical) whether to not respond when status codes
#' in 1xx-3xx series. Default: `FALSE`
#'
#' @section Methods:
#' * `do(response, mssg, template)`:
#'
#'   Execute condition, whether it be message, warning, or error.
#'
#'   * response: is any response from \pkg{crul}, \pkg{curl}, or \pkg{httr}
#'   Execute condition, whether it be message, warning, error, or your
#'   own custom function. This method uses `message_template_verbose`,
#'   and uses it's default value.
#'   * mssg: character string message to include in call. ignored if
#'   template does not have a `message` entry
#'   * template: set a template; by default uses `message_template`
#'
#' * `set_behavior(behavior)`:
#'
#'   * Set behavior, same as setting behavior on initializing with `$new()`
#'
#' @format NULL
#'
#' @seealso [fauxpas::http], [fauxpas::Error-Classes]
#'
#' @examples
#' Error$new()
#' # reset behavior
#' (z <- Error$new())
#' z$set_behavior("warning")
#' z
Error <- R6::R6Class(
  "Error",
  portable = TRUE,
  public = list(
    name = 'HTTP Error',
    behavior = "stop",
    call. = FALSE,
    message_template = NULL,
    message_template_verbose = NULL,
    muffle = FALSE,

    initialize = function(
      behavior = "stop",
      call. = FALSE,
      message_template,
      message_template_verbose,
      muffle = FALSE
    ) {
      stopifnot(inherits(behavior, "character"))
      if (!behavior %in% c('stop', 'warning', 'message')) {
        stop(
          "'behavior' must be one of stop, warning, or message",
          call. = FALSE
        )
      }
      self$behavior <- behavior
      private$behavior_type <- switch(
        self$behavior,
        stop = "error",
        warning = "warning",
        message = "message"
      )

      if (!missing(call.)) {
        self$call. <- call.
      }

      if (!missing(message_template)) {
        if (!is.null(message_template)) {
          if (!inherits(message_template, "character")) {
            stop("'message_template' must be of class character", call. = FALSE)
          }
          self$message_template <- message_template
        }
      }
      if (missing(message_template)) {
        self$message_template <- "{{reason}} (HTTP {{status}})"
      }

      if (!missing(message_template_verbose)) {
        if (!is.null(message_template_verbose)) {
          if (!inherits(message_template_verbose, "character")) {
            stop(
              "'message_template_verbose' must be of class character",
              call. = FALSE
            )
          }
          self$message_template_verbose <- message_template_verbose
        }
      }
      if (missing(message_template_verbose)) {
        self$message_template_verbose <- "{{reason}} (HTTP {{status}}).\n - {{message}}"
      }

      self$muffle <- muffle
    },

    print = function(...) {
      cat(sprintf("<%s>", self$name), sep = "\n")
      cat(paste0("  behavior: ", self$behavior), sep = "\n")
      cat(
        paste0(
          "  message_template: ",
          gsub("\n", "\\\\n", self$message_template)
        ),
        sep = "\n"
      )
      cat(
        paste0(
          "  message_template_verbose: ",
          gsub("\n", "\\\\n", self$message_template_verbose)
        ),
        sep = "\n"
      )
      invisible()
    },

    do = function(response, mssg = "", template = self$message_template) {
      call <- if (self$call.) sys.call(-1) else NULL
      stat <- private$fetch_status(response)
      if (self$muffle) {
        if (stat < 300) return(invisible(response))
      }
      eval(parse(text = self$behavior))(
        private$make_condition(
          response,
          private$behavior_type,
          call,
          mssg,
          template
        )
      )
    },

    set_behavior = function(behavior) {
      stopifnot(inherits(behavior, "character"))
      if (!behavior %in% c('stop', 'warning', 'message')) {
        stop(
          "'behavior' must be one of stop, warning, or message",
          call. = FALSE
        )
      }
      self$behavior <- behavior
      # and set behavior_type
      private$behavior_type <- switch(
        self$behavior,
        stop = "error",
        warning = "warning",
        message = "message"
      )
    }
  ),

  private = list(
    behavior_type = NULL,
    make_condition = function(x, type, call, mssg, template) {
      status <- private$fetch_status(x)
      reason <- httpcode::http_code(status)$message
      xx <- list(reason = reason, status = status, message = mssg)
      message <- whisker::whisker.render(template, xx)
      status_type <- (status %/% 100) * 100
      http_class <- paste0("http_", unique(c(status, status_type, "error")))
      structure(
        list(message = message, call = call),
        class = c(http_class, type, "condition")
      )
    },

    fetch_status = function(x) {
      switch(
        class(x)[1],
        response = x$status_code, # httr
        Response = x$status_code, # webmockr
        VcrResponse = x$stats$status_code, # vcr
        HttpResponse = x$status_code, # crul
        list = x$status_code # curl
      )
    }
  )
)

fetch_status <- function(x) {
  switch(
    class(x)[1],
    response = x$status_code, # httr
    Response = x$status_code, # webmockr
    VcrResponse = x$stats$status_code, # vcr
    HttpResponse = x$status_code, # crul
    list = x$status_code # curl
  )
}

context("Error")

test_that("Error works", {
  skip_on_cran()

  x <- Error$new()
  y <- Error$new(message_template = "{{status}} - foobar - {{reason}}")
  z <- Error$new(message_template_verbose = "{{status}} - {{message}}")

  expect_is(x, "Error")
  expect_is(y, "Error")
  expect_is(z, "Error")

  expect_is(x$behavior, "character")
  expect_is(y$behavior, "character")
  expect_is(z$behavior, "character")

  # the default
  ## simple message
  expect_is(x$message_template, "character")
  expect_match(x$message_template, "HTTP")
  expect_true(grepl("HTTP", x$message_template))
  expect_true(grepl("status", x$message_template))
  expect_true(grepl("reason", x$message_template))
  expect_false(grepl("message", x$message_template))
  ## verbose message
  expect_is(x$message_template_verbose, "character")
  expect_true(grepl("HTTP", x$message_template_verbose))
  expect_true(grepl("status", x$message_template_verbose))
  expect_true(grepl("reason", x$message_template_verbose))
  expect_true(grepl("message", x$message_template_verbose))

  # custom message_template
  expect_is(y$message_template, "character")
  expect_match(y$message_template, "status")
  expect_match(y$message_template, "reason")
  expect_match(y$message_template, "- foobar -")
  expect_false(grepl("HTTP", y$message_template))

  # custom message_template_verbose
  expect_is(z$message_template_verbose, "character")
  expect_match(z$message_template_verbose, "status")
  expect_match(z$message_template_verbose, "message")
  expect_false(grepl("HTTP", z$message_template_verbose))
  expect_false(grepl("reason", z$message_template_verbose))
})


test_that("Error - can set behavior", {
  skip_on_cran()

  x <- Error$new()
  expect_is(x, "Error")
  expect_equal(x$behavior, "stop")
  expect_equal(x$.__enclos_env__$private$behavior_type, "error")

  # reset behavior
  x$set_behavior("message")
  expect_is(x, "Error")
  expect_equal(x$behavior, "message")
  expect_equal(x$.__enclos_env__$private$behavior_type, "message")
})

test_that("Error fails well", {
  skip_on_cran()

  expect_error(Error$new(name = "asdfdf"), "unused argument")

  expect_error(Error$new(a = 5), "unused argument")

  expect_error(Error$new(behavior = 5), "is not TRUE")

  expect_error(Error$new(behavior = "asdfadfs"),
               "'behavior' must be one of stop, warning, or message")
})

---
title: "Introduction to the fauxpas package"
author: "Scott Chamberlain"
date: "`r Sys.Date()`"
output:
  html_document:
    toc: true
    toc_float: true
    theme: readable
vignette: >
  %\VignetteIndexEntry{Introduction to the fauxpas package}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r echo=FALSE}
knitr::opts_chunk$set(
  comment = "#>",
  collapse = TRUE,
  warning = FALSE,
  message = FALSE
)
```

`fauxpas` does http errors!

* HTTP error classes more in line with Ruby/Python/Etc. (psssttt: that is a good thing)
* An error class for each HTTP status in case a user wants to
be specific to an HTTP status code, and general purpose handlers
for any error
* Work with any of the major R http clients: `crul`, `curl`, `httr` (maybe
`RCurl` later)
* Provide flexiblity for what to do on an HTTP error, including
custom functions and message templates

To see example of `fauxpas` usage in other packages, check out the code within any of:

- [nneo](https://github.com/ropenscilabs/nneo)
- [rorcid](https://github.com/ropensci/rorcid)

There are __a lot__ of functions in this package. Don't be scared away by that. 
There are a few major sets of functions that are easy to understand and use:

- There's a function `http()` that is a general purpose function that handles any 
HTTP status codes. There's also a set of functions that follow the pattern `http*()`
where `*` is a HTTP status code (e.g. `http201()`). You can use the general 
purpose function `http()` or the one specific to the status code you are interested
in.
- There's a lower level `R6` class `Error` that is wrapped by `http()` that like 
`http()` is a general purpose function that handles any HTTP status codes. 
There's also a set of `R6` classes that follow the pattern `HTTP*()`
where `*` is a HTTP name (e.g. `HTTPGatewayTimeout`). You can use the general 
purpose class `Error()` or the one specific to the status you are interested
in.

`fauxpas` uses [httpcode][] under the hood - which holds all the info about HTTP status 
codes. 

## Install

CRAN version

```{r eval=FALSE}
install.packages("fauxpas")
```

Dev version

```{r eval=FALSE}
# install.packages("remotes")
remotes::install_github("ropensci/fauxpas")
```

```{r}
library("fauxpas")
```

## Find classes for a HTTP status code

When using `fauxpas` in another package I personally like to use the `HTTP*` classes.

Since these are named with their HTTP reason (e.g. `HTTPGatewayTimeout`), it's not super
straightforward to find them with a status code, which is what one always has as a response
from a server. A good way to find these is with a new function `find_error_class()`. Just 
pass the function a HTTP status code and it will find the matching class. 

```{r}
(x <- find_error_class(418))
```

Which returns the matching error class object, with which you can initialize a new object:

```{r}
x$new()
```

If you pass it a status code it doesn't know about it errors

```{r eval=FALSE}
find_error_class(999)
#> Error in find_error_class(999) : no method found for 999
```


## Use fauxpas with HTTP clients

### crul

```{r eval=FALSE}
library("crul")
cli <- HttpClient$new("https://httpbin.org/status/414")
res <- cli$get()
http(res)
#> Error: Request-URI Too Long (HTTP 414).
http414(res)
#> Error: Request-URI Too Long (HTTP 414).
```

```{r eval=FALSE}
x <- HTTPRequestURITooLong$new()
x$do_verbose(res)
#> Error: Request-URI Too Long (HTTP 414).
#> - The server is refusing to service the request because the Request-URI is
#>    longer than the server is willing to interpret. This rare condition is only likely
#>    to occur when a client has improperly converted a POST request to a GET request
#>    with long query information, when the client has descended into a URI black hole
#>    of redirection (e.g., a redirected URI prefix that points to a suffix of itself),
#>    or when the server is under attack by a client attempting to exploit security
#>    holes present in some servers using fixed-length buffers for reading or
#>    manipulating the Request-URI.
```

### curl

```{r eval=FALSE}
library("curl")
h <- curl::new_handle()
curl::handle_setopt(h)
resp <- curl::curl_fetch_memory("https://httpbin.org/status/404", h)
http(resp)
#> Error: Not Found (HTTP 404).
http404(resp)
#> Error: Not Found (HTTP 404).
```

```{r eval=FALSE}
x <- HTTPNotFound$new()
x$do_verbose(resp)
#> Error:  Not Found (HTTP 404).
#>  - The server has not found anything matching the Request-URI. No indication is
#> given of whether the condition is temporary or permanent. The 410 (Gone) status
#> code SHOULD be used if the server knows, through some internally configurable
#> mechanism, that an old resource is permanently unavailable and has no forwarding
#> address. #> This status code is commonly used when the server does not wish to
#> reveal exactly why the request has been refused, or when no other response is
#> applicable.
```

### httr

```{r eval=FALSE}
library("httr")
res <- GET("https://httpbin.org/status/405")
http405(res)
#> Error: Method Not Allowed (HTTP 405).
```

```{r eval=FALSE}
x <- HTTPMethodNotAllowed$new()
x$do_verbose(res)
#> Error: Method Not Allowed (HTTP 405).
#>  - The method specified in the Request-Line is not allowed for the resource
#> identified by the Request-URI. The response MUST include an Allow header
#> containing a list of valid methods for the requested resource.
```

[httpcode]: https://github.com/sckott/httpcode

---
title: "Introduction to the fauxpas package"
author: "Scott Chamberlain"
date: "`r Sys.Date()`"
output:
  html_document:
    toc: true
    toc_float: true
    theme: readable
vignette: >
  %\VignetteIndexEntry{Introduction to the fauxpas package}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r echo=FALSE}
knitr::opts_chunk$set(
  comment = "#>",
  collapse = TRUE,
  warning = FALSE,
  message = FALSE
)
```

`fauxpas` does http errors!

* HTTP error classes more in line with Ruby/Python/Etc. (psssttt: that is a good thing)
* An error class for each HTTP status in case a user wants to
be specific to an HTTP status code, and general purpose handlers
for any error
* Work with any of the major R http clients: `crul`, `curl`, `httr` (maybe
`RCurl` later)
* Provide flexiblity for what to do on an HTTP error, including
custom functions and message templates

There are __a lot__ of functions in this package. Don't be scared away by that. 
There are a few major sets of functions that are easy to understand and use:

- There's a function `http()` that is a general purpose function that handles any 
HTTP status codes. There's also a set of functions that follow the pattern `http*()`
where `*` is a HTTP status code (e.g. `http201()`). You can use the general 
purpose function `http()` or the one specific to the status code you are interested
in.
- There's a lower level `R6` class `Error` that is wrapped by `http()` that like 
`http()` is a general purpose function that handles any HTTP status codes. 
There's also a set of `R6` classes that follow the pattern `HTTP*()`
where `*` is a HTTP name (e.g. `HTTPGatewayTimeout`). You can use the general 
purpose class `Error()` or the one specific to the status you are interested
in.

`fauxpas` uses [httpcode][] under the hood - which holds all the info about HTTP status 
codes. 

## Install

CRAN version

```{r eval=FALSE}
install.packages("fauxpas")
```

Dev version

```{r eval=FALSE}
# install.packages("remotes")
remotes::install_github("sckott/fauxpas")
```

```{r}
library("fauxpas")
```

## Find classes for a HTTP status code

When using `fauxpas` in another package I personally like to use the `HTTP*` classes.

Since these are named with their HTTP reason (e.g. `HTTPGatewayTimeout`), it's not super
straightforward to find them with a status code, which is what one always has as a response
from a server. A good way to find these is with a new function `find_error_class()`. Just 
pass the function a HTTP status code and it will find the matching class. 

```{r}
(x <- find_error_class(418))
```

Which returns the matching error class object, with which you can initialize a new object:

```{r}
x$new()
```

If you pass it a status code it doesn't know about it errors

```{r eval=FALSE}
find_error_class(999)
#> Error in find_error_class(999) : no method found for 999
```


## Use fauxpas with HTTP clients

### crul

```{r eval=FALSE}
library("crul")
cli <- HttpClient$new("https://httpbin.org/status/414")
res <- cli$get()
http(res)
#> Error: Request-URI Too Long (HTTP 414).
http414(res)
#> Error: Request-URI Too Long (HTTP 414).
```

```{r eval=FALSE}
x <- HTTPRequestURITooLong$new()
x$do_verbose(res)
#> Error: Request-URI Too Long (HTTP 414).
#> - The server is refusing to service the request because the Request-URI is
#>    longer than the server is willing to interpret. This rare condition is only likely
#>    to occur when a client has improperly converted a POST request to a GET request
#>    with long query information, when the client has descended into a URI black hole
#>    of redirection (e.g., a redirected URI prefix that points to a suffix of itself),
#>    or when the server is under attack by a client attempting to exploit security
#>    holes present in some servers using fixed-length buffers for reading or
#>    manipulating the Request-URI.
```

### curl

```{r eval=FALSE}
library("curl")
h <- curl::new_handle()
curl::handle_setopt(h)
resp <- curl::curl_fetch_memory("https://httpbin.org/status/404", h)
http(resp)
#> Error: Not Found (HTTP 404).
http404(resp)
#> Error: Not Found (HTTP 404).
```

```{r eval=FALSE}
x <- HTTPNotFound$new()
x$do_verbose(resp)
#> Error:  Not Found (HTTP 404).
#>  - The server has not found anything matching the Request-URI. No indication is
#> given of whether the condition is temporary or permanent. The 410 (Gone) status
#> code SHOULD be used if the server knows, through some internally configurable
#> mechanism, that an old resource is permanently unavailable and has no forwarding
#> address. #> This status code is commonly used when the server does not wish to
#> reveal exactly why the request has been refused, or when no other response is
#> applicable.
```

### httr

```{r eval=FALSE}
library("httr")
res <- GET("https://httpbin.org/status/405")
http405(res)
#> Error: Method Not Allowed (HTTP 405).
```

```{r eval=FALSE}
x <- HTTPMethodNotAllowed$new()
x$do_verbose(res)
#> Error: Method Not Allowed (HTTP 405).
#>  - The method specified in the Request-Line is not allowed for the resource
#> identified by the Request-URI. The response MUST include an Allow header
#> containing a list of valid methods for the requested resource.
```

[httpcode]: https://github.com/sckott/httpcode

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fauxpas-package.R
\docType{package}
\name{fauxpas-package}
\alias{fauxpas}
\alias{fauxpas-package}
\title{fauxpas: HTTP Error Helpers}
\description{
HTTP error helpers. Methods included for general purpose HTTP error handling, as well as individual methods for every HTTP status code, both via status code numbers as well as their descriptive names. Supports ability to adjust behavior to stop, message or warning. Includes ability to use custom whisker template to have any configuration of status code, short description, and verbose message. Currently supports integration with 'crul', 'curl', and 'httr'.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://sckott.github.io/fauxpas/}
  \item \url{https://github.com/sckott/fauxpas}
  \item Report bugs at \url{https://github.com/sckott/fauxpas/issues}
}

}
\author{
\strong{Maintainer}: Scott Chamberlain \email{myrmecocystus@gmail.com} (\href{https://orcid.org/0000-0003-1444-9135}{ORCID})

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/errors.R
\name{Error}
\alias{Error}
\title{Error class}
\arguments{
\item{behavior}{Behavior of the error. default: auto. See Details}

\item{message_template}{A message template. optional. use whisker
templating. names to use are: reason and status. use in template
like \code{\{\{reason\}\}} and  \code{\{\{status\}\}}. Note that
\code{\{\{message\}\}} that is used in \code{message_template_verbose}
will be ignored here.}

\item{call.}{(logical) indicating if the call should become part
of the error message. Default: \code{FALSE}}

\item{message_template_verbose}{A verbose message template. optional.
use whisker templating. names to use are: reason, status, message.
use in template like \code{{{reason}}}, \code{{{status}}}, and
\code{{{message}}}. Note that this is ignored here, but is
used in the \verb{HTTP*} methods (e.g. \code{HTTPBadRequest})}

\item{muffle}{(logical) whether to not respond when status codes
in 1xx-3xx series. Default: \code{FALSE}}
}
\description{
Error class
}
\section{behavior parameter options}{

\itemize{
\item stop - use \code{stop}
\item warning - use \code{warning}
\item message - use \code{message}
\item auto - toggle between \code{stop} and \code{message} depending
on the HTTP status code series. Defaults will be:
\itemize{
\item 1xx: \code{message}
\item 2xx: \code{message}
\item 3xx: \code{message}
\item 4xx: \code{stop}
\item 5xx: \code{stop}
}
}

Of course, you can always override the defaults.
}

\section{Methods}{

\itemize{
\item \code{do(response, mssg, template)}:

Execute condition, whether it be message, warning, or error.
\itemize{
\item response: is any response from \pkg{crul}, \pkg{curl}, or \pkg{httr}
Execute condition, whether it be message, warning, error, or your
own custom function. This method uses \code{message_template_verbose},
and uses it's default value.
\item mssg: character string message to include in call. ignored if
template does not have a \code{message} entry
\item template: set a template; by default uses \code{message_template}
}
\item \code{set_behavior(behavior)}:
\itemize{
\item Set behavior, same as setting behavior on initializing with \verb{$new()}
}
}
}

\examples{
Error$new()
# reset behavior
(z <- Error$new())
z$set_behavior("warning")
z
}
\seealso{
\link{http}, \link{Error-Classes}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/find_error_class.R
\name{find_error_class}
\alias{find_error_class}
\title{Find error classes}
\usage{
find_error_class(status_code)
}
\arguments{
\item{status_code}{(numeric,integer) A status code}
}
\value{
an object of class \code{R6ClassGenerator}. call \code{$new()}
to initialize a new instance
}
\description{
Find error classes
}
\examples{
find_error_class(414)
find_error_class(418)
find_error_class(505)

# initialize the class
find_error_class(418)$new()

# not found
\dontrun{find_error_class(999)}
}
\seealso{
\code{\link[fauxpas]{Error}}, \code{\link[fauxpas]{Error-Classes}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/errors-children.R
\name{Error-Classes}
\alias{Error-Classes}
\alias{HTTPContinue}
\alias{HTTPSwitchProtocol}
\alias{HTTPProcessing}
\alias{HTTPOK}
\alias{HTTPCreated}
\alias{HTTPAccepted}
\alias{HTTPNonAuthoritativeInformation}
\alias{HTTPNoContent}
\alias{HTTPResetContent}
\alias{HTTPPartialContent}
\alias{HTTPMultiStatus}
\alias{HTTPAlreadyReported}
\alias{HTTPImUsed}
\alias{HTTPMultipleChoices}
\alias{HTTPMovedPermanently}
\alias{HTTPFound}
\alias{HTTPSeeOther}
\alias{HTTPNotModified}
\alias{HTTPUseProxy}
\alias{HTTPSwitchProxy}
\alias{HTTPTemporaryRedirect}
\alias{HTTPPermanentRedirect}
\alias{HTTPBadRequest}
\alias{HTTPUnauthorized}
\alias{HTTPPaymentRequired}
\alias{HTTPForbidden}
\alias{HTTPNotFound}
\alias{HTTPMethodNotAllowed}
\alias{HTTPNotAcceptable}
\alias{HTTPProxyAuthenticationRequired}
\alias{HTTPRequestTimeout}
\alias{HTTPConflict}
\alias{HTTPGone}
\alias{HTTPLengthRequired}
\alias{HTTPPreconditionFailed}
\alias{HTTPRequestEntityTooLarge}
\alias{HTTPRequestURITooLong}
\alias{HTTPUnsupportedMediaType}
\alias{HTTPRequestRangeNotSatisfiable}
\alias{HTTPExpectationFailed}
\alias{HTTPTeaPot}
\alias{HTTPAuthenticationTimeout}
\alias{HTTPMethodFailure}
\alias{HTTPMisdirectedRequest}
\alias{HTTPUnprocessableEntity}
\alias{HTTPLocked}
\alias{HTTPFailedDependency}
\alias{HTTPUnorderedCollection}
\alias{HTTPUpgradeRequired}
\alias{HTTPPreconditionRequired}
\alias{HTTPTooManyRequests}
\alias{HTTPRequestHeaderFieldsTooLarge}
\alias{HTTPLoginTimeout}
\alias{HTTPNoResponse}
\alias{HTTPRetryWith}
\alias{HTTPBlockedByWindowsParentalControls}
\alias{HTTPUnavailableForLegalReasons}
\alias{HTTPRequestHeaderTooLarge}
\alias{HTTPCertError}
\alias{HTTPNoCert}
\alias{HTTPHTTPToHTTPS}
\alias{HTTPTokenExpiredInvalid}
\alias{HTTPClientClosedRequest}
\alias{HTTPInternalServerError}
\alias{HTTPNotImplemented}
\alias{HTTPBadGateway}
\alias{HTTPServiceUnavailable}
\alias{HTTPGatewayTimeout}
\alias{HTTPHTTPVersionNotSupported}
\alias{HTTPVariantAlsoNegotiates}
\alias{HTTPInsufficientStorage}
\alias{HTTPLoopDetected}
\alias{HTTPBandwidthLimitExceeded}
\alias{HTTPNotExtended}
\alias{HTTPNetworkAuthenticationRequired}
\alias{HTTPNetworkReadTimeoutError}
\alias{HTTPNetworkConnectTimeoutError}
\alias{HTTPWebServerReturnedUnknownError}
\alias{HTTPWebServerIsDown}
\alias{HTTPConnectionTimedOut}
\alias{HTTPOriginIsUnreachable}
\alias{HTTPATimeoutOccurred}
\alias{HTTPSSLHandshakeFailed}
\alias{HTTPInvalidSSLCertificate}
\alias{HTTPRailgunError}
\title{Individual error classes}
\description{
These error classes are for each HTTP error, and inherit from
the \code{\link{Error}} class in this package.
}
\details{
In addition to what's available in \code{\link{Error}},
these classes have a single variable \code{mssg} that is the very
verbose complete message describing the HTTP condition in detail.
You can include that message in your condition by using \code{do_verbose}
(see below)

\strong{Methods}

In addition to the methods documented in \code{\link{Error}}, these methods
also have:
\itemize{
  \item \code{do_verbose(response, template)} {

  Execute condition, whether it be message, warning, or error.

  \itemize{
   \item response: is any response from \pkg{crul}, \pkg{curl}, or \pkg{httr}
  Execute condition, whether it be message, warning, error, or your
  own custom function. This method uses \code{message_template_verbose},
  and uses it's default value.
   \item template: a template to use for the verbose message, see \code{\link{Error}}
  for details
  }
  }
}
}
\examples{
HTTPRequestURITooLong$new(
  message_template = "{{reason}} ............ {{status}}",
  message_template_verbose = "{{reason}} .>.>.>.>.>.> {{status}}\n {{message}}"
)
}
\seealso{
\code{\link[fauxpas]{Error}}, \code{\link[fauxpas]{http}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/error_http_generated.R, R/s3-classes.R
\name{http100}
\alias{http100}
\alias{http101}
\alias{http102}
\alias{http200}
\alias{http201}
\alias{http202}
\alias{http203}
\alias{http204}
\alias{http205}
\alias{http206}
\alias{http207}
\alias{http208}
\alias{http226}
\alias{http300}
\alias{http301}
\alias{http302}
\alias{http303}
\alias{http304}
\alias{http305}
\alias{http306}
\alias{http307}
\alias{http308}
\alias{http400}
\alias{http401}
\alias{http402}
\alias{http403}
\alias{http404}
\alias{http405}
\alias{http406}
\alias{http407}
\alias{http408}
\alias{http409}
\alias{http410}
\alias{http411}
\alias{http412}
\alias{http413}
\alias{http414}
\alias{http415}
\alias{http416}
\alias{http417}
\alias{http418}
\alias{http419}
\alias{http420}
\alias{http422}
\alias{http423}
\alias{http424}
\alias{http425}
\alias{http426}
\alias{http428}
\alias{http429}
\alias{http431}
\alias{http440}
\alias{http444}
\alias{http449}
\alias{http450}
\alias{http451}
\alias{http494}
\alias{http495}
\alias{http496}
\alias{http497}
\alias{http498}
\alias{http499}
\alias{http500}
\alias{http501}
\alias{http502}
\alias{http503}
\alias{http504}
\alias{http505}
\alias{http506}
\alias{http507}
\alias{http508}
\alias{http509}
\alias{http510}
\alias{http511}
\alias{http598}
\alias{http599}
\alias{http}
\title{higher level error wrappers}
\usage{
http100(response, behavior = "auto", message_template, muffle = FALSE)

http101(response, behavior = "auto", message_template, muffle = FALSE)

http102(response, behavior = "auto", message_template, muffle = FALSE)

http200(response, behavior = "auto", message_template, muffle = FALSE)

http201(response, behavior = "auto", message_template, muffle = FALSE)

http202(response, behavior = "auto", message_template, muffle = FALSE)

http203(response, behavior = "auto", message_template, muffle = FALSE)

http204(response, behavior = "auto", message_template, muffle = FALSE)

http205(response, behavior = "auto", message_template, muffle = FALSE)

http206(response, behavior = "auto", message_template, muffle = FALSE)

http207(response, behavior = "auto", message_template, muffle = FALSE)

http208(response, behavior = "auto", message_template, muffle = FALSE)

http226(response, behavior = "auto", message_template, muffle = FALSE)

http300(response, behavior = "auto", message_template, muffle = FALSE)

http301(response, behavior = "auto", message_template, muffle = FALSE)

http302(response, behavior = "auto", message_template, muffle = FALSE)

http303(response, behavior = "auto", message_template, muffle = FALSE)

http304(response, behavior = "auto", message_template, muffle = FALSE)

http305(response, behavior = "auto", message_template, muffle = FALSE)

http306(response, behavior = "auto", message_template, muffle = FALSE)

http307(response, behavior = "auto", message_template, muffle = FALSE)

http308(response, behavior = "auto", message_template, muffle = FALSE)

http400(response, behavior = "auto", message_template, muffle = FALSE)

http401(response, behavior = "auto", message_template, muffle = FALSE)

http402(response, behavior = "auto", message_template, muffle = FALSE)

http403(response, behavior = "auto", message_template, muffle = FALSE)

http404(response, behavior = "auto", message_template, muffle = FALSE)

http405(response, behavior = "auto", message_template, muffle = FALSE)

http406(response, behavior = "auto", message_template, muffle = FALSE)

http407(response, behavior = "auto", message_template, muffle = FALSE)

http408(response, behavior = "auto", message_template, muffle = FALSE)

http409(response, behavior = "auto", message_template, muffle = FALSE)

http410(response, behavior = "auto", message_template, muffle = FALSE)

http411(response, behavior = "auto", message_template, muffle = FALSE)

http412(response, behavior = "auto", message_template, muffle = FALSE)

http413(response, behavior = "auto", message_template, muffle = FALSE)

http414(response, behavior = "auto", message_template, muffle = FALSE)

http415(response, behavior = "auto", message_template, muffle = FALSE)

http416(response, behavior = "auto", message_template, muffle = FALSE)

http417(response, behavior = "auto", message_template, muffle = FALSE)

http418(response, behavior = "auto", message_template, muffle = FALSE)

http419(response, behavior = "auto", message_template, muffle = FALSE)

http420(response, behavior = "auto", message_template, muffle = FALSE)

http422(response, behavior = "auto", message_template, muffle = FALSE)

http423(response, behavior = "auto", message_template, muffle = FALSE)

http424(response, behavior = "auto", message_template, muffle = FALSE)

http425(response, behavior = "auto", message_template, muffle = FALSE)

http426(response, behavior = "auto", message_template, muffle = FALSE)

http428(response, behavior = "auto", message_template, muffle = FALSE)

http429(response, behavior = "auto", message_template, muffle = FALSE)

http431(response, behavior = "auto", message_template, muffle = FALSE)

http440(response, behavior = "auto", message_template, muffle = FALSE)

http444(response, behavior = "auto", message_template, muffle = FALSE)

http449(response, behavior = "auto", message_template, muffle = FALSE)

http450(response, behavior = "auto", message_template, muffle = FALSE)

http451(response, behavior = "auto", message_template, muffle = FALSE)

http494(response, behavior = "auto", message_template, muffle = FALSE)

http495(response, behavior = "auto", message_template, muffle = FALSE)

http496(response, behavior = "auto", message_template, muffle = FALSE)

http497(response, behavior = "auto", message_template, muffle = FALSE)

http498(response, behavior = "auto", message_template, muffle = FALSE)

http499(response, behavior = "auto", message_template, muffle = FALSE)

http500(response, behavior = "auto", message_template, muffle = FALSE)

http501(response, behavior = "auto", message_template, muffle = FALSE)

http502(response, behavior = "auto", message_template, muffle = FALSE)

http503(response, behavior = "auto", message_template, muffle = FALSE)

http504(response, behavior = "auto", message_template, muffle = FALSE)

http505(response, behavior = "auto", message_template, muffle = FALSE)

http506(response, behavior = "auto", message_template, muffle = FALSE)

http507(response, behavior = "auto", message_template, muffle = FALSE)

http508(response, behavior = "auto", message_template, muffle = FALSE)

http509(response, behavior = "auto", message_template, muffle = FALSE)

http510(response, behavior = "auto", message_template, muffle = FALSE)

http511(response, behavior = "auto", message_template, muffle = FALSE)

http598(response, behavior = "auto", message_template, muffle = FALSE)

http599(response, behavior = "auto", message_template, muffle = FALSE)

http(response, behavior = "auto", message_template, muffle = FALSE)
}
\arguments{
\item{response}{The result of a call via \pkg{crul}, \pkg{curl}, or
\pkg{httr}}

\item{behavior}{Behavior of the error. default: auto. See Details}

\item{message_template}{A message template. optional. use whisker
templating. names to use are: reason and status. use in template
like \code{\{\{reason\}\}} and  \code{\{\{status\}\}}. Note that
\code{\{\{message\}\}} that is used in \code{message_template_verbose}
will be ignored here.}

\item{muffle}{(logical) whether to not respond when status codes
in 1xx-3xx series. Default: \code{FALSE}}
}
\description{
higher level error wrappers
}
\note{
These \code{http*} methods only use \code{$do} and not
\code{$do_verbose}.
}
\section{behavior parameter options}{

\itemize{
 \item stop - use \code{stop}
 \item warning - use \code{warning}
 \item message - use \code{message}
 \item auto - toggle between \code{stop} and \code{message} depending
 on the HTTP status code series. Defaults will be:
 \itemize{
  \item 1xx: \code{message}
  \item 2xx: \code{message}
  \item 3xx: \code{message}
  \item 4xx: \code{stop}
  \item 5xx: \code{stop}
 }
}

Of course, you can always override the defaults.
}

\section{using package \pkg{curl}}{

curl reponses are simple lists, so we have little to go on to make sure
it's a response from the \pkg{curl} package. We check for list names
internally but of course you could pass in a list with the right named
elements, while the values are complete nonsense, in which case
we'll probably fail badly. There's not much we can do.
}

\examples{
if (requireNamespace("crul")) {
 library("crul")
 res <- HttpClient$new("https://httpbin.org/status/418")$get()
 \dontrun{http(res)}
 http(res, behavior = "warning")
 http(res, behavior = "message")

 res <- HttpClient$new("https://httpbin.org/status/414")$get()
 \dontrun{http414(res)}
 http(res, behavior = "warning")
 http(res, behavior = "message")

 res <- HttpClient$new("https://httpbin.org/asdfafadsf")$get()
 \dontrun{http404(res)}
 http(res, behavior = "warning")
 http(res, behavior = "message")
}

if (requireNamespace("curl")) {
 library("curl")
 h <- curl::new_handle()
 curl::handle_setopt(h)
 res <- curl::curl_fetch_memory("https://httpbin.org/status/418", h)
 \dontrun{http(res)}
 http(res, behavior = "warning")
 http(res, behavior = "message")
}

if (requireNamespace("httr")) {
 library("httr")
 res <- GET("https://httpbin.org/status/418")
 \dontrun{http(res)}
 http(res, behavior = "warning")
 http(res, behavior = "message")
}

# muffle responses
if (requireNamespace("crul")) {
 library("crul")
 res201 <- HttpClient$new("https://httpbin.org")$get("status/201")
 res404 <- HttpClient$new("https://httpbin.org")$get("status/404")
 # status codes < 300 CAN be muffled - i.e., return the http response object
 http(res201, muffle = TRUE)
 # status codes > 300 CAN NOT be muffled - i.e., return the http response object
 \dontrun{
 http(res404, muffle = TRUE)
 }
}
}
\seealso{
\code{\link{Error}}, \code{\link[fauxpas]{Error-Classes}}
}
